package CH.ifa.draw.application;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;

public class DrawingViewFrame extends Frame {

  private StandardDrawingView drawingView;
  private ScrollPane sp;
  private Dimension defSize;
  //private int viewNumber;
  //private static int viewCounter = 0;

  private boolean onDiscardReleaseDrawing=false;

  public DrawingViewFrame(DrawApplication editor,
    StandardDrawingView drawingView, Drawing drawing)
  {
        super();
	Image menuFrameImage = editor.menuFrame().getIconImage();
	if (menuFrameImage != null) {
	  Image iconImage = Toolkit.getDefaultToolkit()
            .createImage(menuFrameImage.getSource());
	  // Is the next check still required?
	  if (iconImage != null)
            setIconImage(iconImage);
	}
        this.drawingView = drawingView;
	//viewNumber = ++viewCounter;
        addWindowListener(editor);
        sp = new ScrollPane();
        Adjustable vadjust = sp.getVAdjustable();
        Adjustable hadjust = sp.getHAdjustable();
        hadjust.setUnitIncrement(16);
        vadjust.setUnitIncrement(16);
        sp.add(drawingView);
        add(sp);
        Point pos = editor.menuFrame.getLocationOnScreen();
        Dimension siz = editor.menuFrame.getSize();
	defSize = new Dimension(siz.width, 600);
        setBounds(pos.x, pos.y + siz.height,defSize.width,defSize.height);
	doLayout();
        drawingView.setDrawing(drawing);
        if (editor.tool()==null) {
           editor.toolDone();
        }
  	setDrawing(drawing);
  }

  public DrawingView view() {
    DrawingView view=drawingView;
    if (view==null) {
      view=NullDrawingView.INSTANCE;
    }
    return view;
  }

  void setDrawing(Drawing drawing) {
    drawingView.setDrawing(drawing);
    setTitle(drawing.getName()); //+"["+viewNumber+"]");
    
   //  FigureEnumeration k = drawing.figures();
//     Rectangle minr = null;
//     while (k.hasMoreElements()) {
//       Rectangle r = k.nextFigure().displayBox();
//       if (minr == null)
// 	minr = r;
//       else
// 	minr = minr.union(r);
//     }
    if (!isShowing())
      show(); // has to be on screen to determine ViewportSize
    if (drawing.figures().hasMoreElements() && drawing instanceof Figure) {
      Dimension frame = getSize();
      Dimension inner = sp.getViewportSize();
      Dimension border = new Dimension(frame.width-inner.width,
				       frame.height-inner.height);
      Rectangle minr = ((Figure)drawing).displayBox();
      //setSize(minr.width+border.width,
      //        minr.height+border.height);
      // changed to:
      setSize(minr.x+minr.width+border.width+16,
              minr.y+minr.height+border.height+16);
      validate();
      //sp.setScrollPosition(minr.x, minr.y);
    } else {
      setSize(defSize.width, defSize.height);
      validate();
      //sp.setScrollPosition(64,64);
    }
  }

  public void onDiscardRelease() {
    onDiscardReleaseDrawing=true;
  }

  // This method cleans up the frame and removes all references
  // to other objects. Frames are not always garbage collected
  // correctly, therefore we must make sure to be as lightweight
  // as possible. This method is not named dispose(), because
  // a dispose call still leave us with a functionally intact object.
  // This call will give up the frame once and for all.
  void discard() {
    setVisible(false);
    sp.remove(drawingView);
    this.remove(sp);
    if (onDiscardReleaseDrawing) {
      drawingView.drawing().release();
    }
    drawingView.setDrawing(NullDrawing.INSTANCE);
    drawingView=null;
    sp=null;
    defSize=null;
    // Maybe dispose has already been called, but let's be safe.
    dispose();
  }
}

