/*
 * @(#)ArcConnection.java 5.1
 *
 */

package de.renew.gui;

import java.awt.*;
import java.util.*;
import java.io.IOException;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.standard.*;
import de.renew.simulator.*;
import de.renew.shadow.*;

public class ArcConnection extends LineConnection implements InscribableFigure {
    /*
     * Serialization support.
     */
    private static final long serialVersionUID = -7959501008698525009L;
    private int arcConnectionSerializedDataVersion = 1;

    public static final ArcConnection NormalArc
                        = new ArcConnection(ShadowArc.ordinary);
    public static final ArcConnection TestArc
                        = new ArcConnection(ShadowArc.test);
    public static final ArcConnection ReserveArc
                        = new ArcConnection(ShadowArc.both);

    private ShadowArc shadow = null;

    public ArcConnection() {
    }

    public ArcConnection(int arcType) {
      ArrowTip start = null, end = null;
      if (arcType == ShadowArc.both)
	start = new ArrowTip();
      if (arcType != ShadowArc.test)
	end = new ArrowTip();
      setStartDecoration(start);
      setEndDecoration(end);
    }

    public int getArcType() {
      int arrowMode = ((Integer)getAttribute("ArrowMode")).intValue();
      if (arrowMode == ARROW_TIP_NONE)
	return ShadowArc.test;
      if (arrowMode == ARROW_TIP_BOTH)
	return ShadowArc.both;
      return ShadowArc.ordinary;
    }

    public boolean canConnect(Figure start, Figure end) {
        return (start instanceof TransitionFigure
		&& end instanceof PlaceFigure
		 ||
		 start instanceof PlaceFigure
		&& end instanceof TransitionFigure);
    }

  public boolean getTraceMode() {
    Object value=getAttribute("TraceMode");
    if (value instanceof Boolean) {
      return ((Boolean)value).booleanValue();
    }
    return true;
  }

  public void release() {
    super.release();
    if (shadow!=null)
      shadow.discard();
  }  

  public ShadowNetElement buildShadow(ShadowNet net) {
    shadow=new ShadowArc(startShadow(), endShadow(), getArcType());
    shadow.context = this;
    shadow.setTrace(getTraceMode());
    return shadow;
    // No longer calling the syntax check. This is done
    // after editing the text and in no other place should a
    // syntax check occur.
  }

  public boolean isReverse() {
    return ((Integer)getAttribute("ArrowMode")).intValue() == ARROW_TIP_START;
  }

  private ShadowNode anyShadow(boolean end) {
    ShadowHolder holder = (ShadowHolder)(end ? endFigure() : startFigure());
    return (ShadowNode)holder.getShadow();
  }

  private ShadowNode startShadow() {
    return anyShadow(isReverse());
  }

  private ShadowNode endShadow() {
    return anyShadow(!isReverse());
  }

  public ShadowNetElement getShadow() {
    return shadow;
  }

  public String getDefaultInscription() {
     return "x";
  }
 
}
