/*
 * @(#)CPNApplication.java 5.1
 *
 */

package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import CH.ifa.draw.application.*;
//import CH.ifa.draw.contrib.*;
import fs.*;
import de.renew.shadow.*;
import de.renew.simulator.Simulator;
import de.renew.simulator.NetInstance;
import de.renew.simulator.PlaceInstance;

public class CPNApplication extends DrawApplication {
    static private final String CPNIMAGES = "/de/renew/gui/images/";
    static private final Color INSTANCE_COLOR = new Color(200,200,255);
    static private final Color TOKEN_BAG_COLOR = new Color(230,230,255);

    private ToolButton fInscrTB, fNameTB, fDeclTB, fConnTextTB, fTextTB;
    CPNSimulation fSimulation;

    CPNApplication() {
        this(new String[0]);
    }

    CPNApplication(String[] args) {
      super("Reference Net Workshop", ".rnw", args, CPNIMAGES+"RENEW.gif");
    }

  CPNSimulation getSimulation() {
    if (fSimulation==null) {
      String netCompiler = System.getProperty("de.renew.netCompiler");
      ShadowCompiler compiler=null;
      
      if (netCompiler != null) {
	System.err.println("Trying to switch to net compiler "+
	  netCompiler+"...");
	try {
	  compiler = (ShadowCompiler)Class.forName(netCompiler).newInstance();
	} catch (Exception e) {
	  System.err.println("Switch failed:"+e);
	}
      }
      if (compiler == null) {
	System.err.println("Using default net formalism...");
	compiler = new de.renew.formalism.java.JavaNetCompiler();
      } else {
	System.err.println("Using net compiler "+netCompiler+"...");
      }

      fSimulation=new CPNSimulation(compiler);
    }
    return fSimulation;
  }

    protected Drawing createDrawing() {
        return new CPNDrawing();
    }

    protected DrawApplication newWindow() { return new CPNApplication(); }

    /**
     * Creates the standard menus. Clients override this
     * method to add additional menus.
     */
    protected void createMenus(MenuBar mb) {
		mb.add(createFileMenu());
		mb.add(createEditMenu());
		mb.add(createAlignmentMenu());
		mb.add(createAttributesMenu());
		mb.add(createSimulationMenu());
		mb.add(createDrawingsMenu());
    }

    /**
     * Create the simulation menu.
     */
    protected Menu createSimulationMenu() {

	Menu menu = new Menu("Simulation");
	
    	MenuItem mi;

    	mi = new MenuItem("Run simulation", new MenuShortcut('r'));
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		if (!fSimulation.isSimulationActive()) {
		  initSimulation();
		}
		if (fSimulation.isSimulationActive()) {
		  showStatus("Simulation running.");
		  fSimulation.simulationRun();
		}
	      }
	   }
	);
	menu.add(mi);

    	mi = new MenuItem("Simulation Step", new MenuShortcut('i'));
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		new Thread() {
		  public void run() {
		    if (!fSimulation.isSimulationActive()) {
		      initSimulation();
		    } else {
		      showStatus("Searching for a binding...");
		      int status=fSimulation.simulationStep();
		      switch (status) {
		      case Simulator.statusStopped:
			if (fSimulation.isSimulationActive()) {
			  showStatus("Simulation halted.");
			} else {
			  showStatus("Simulation terminated.");
			}
			break;
		      case Simulator.statusStepComplete:
			showStatus("Simulation step completed.");
			break;
		      case Simulator.statusLastComplete:
			showStatus("Simulation step completed. "+
			  "No more enabled bindings.");
			break;
		      case Simulator.statusCurrentlyDisabled:
			showStatus("No enabled bindings found.");
			break;
		      case Simulator.statusDisabled:
			showStatus("No more enabled bindings.");
			break;
		      }
		    }
		  }
		}.start();
	      }
	   }
	);
	menu.add(mi);

    	mi = new MenuItem("Halt simulation", new MenuShortcut('h'));
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		fSimulation.simulationStop();
		showStatus("Simulation halted.");
	      }
	   }
	);
	menu.add(mi);
	
    	mi = new MenuItem("Terminate simulation", new MenuShortcut('t'));
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		fSimulation.simulationTerminate();
		showStatus("Simulation terminated.");
                TransitionInstanceFigure.closeBindingSelectionFrame();
                closeAllSimulationDrawings();
	      }
	   }
	);
	menu.add(mi);

	menu.addSeparator();

	mi = new MenuItem("Syntax check only");
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		if (fSimulation.isSimulationActive()) {
		  showStatus("Terminate the current simulation "+
		    "before a syntax check.");
		} else {
		  try {
		    fSimulation.syntaxCheckOnly();
		    showStatus("Syntax check successful.");
                    closeSyntaxErrorFrame();
		    return;
		  }
		  catch (SyntaxException e) {
		    processSyntaxException(new FigureException(e),true);
		  }
		}
	      }
	   }
	);
	menu.add(mi);

	String[] lintNames=getSimulation().getLintNames();
	for (int i=0;i<lintNames.length;i++) {
	  final int lintNr=i;
	  final String name=lintNames[i];
	  final String upperName=name.substring(0,1).toUpperCase()+
	    name.substring(1);

	  mi = new MenuItem(upperName);
	  mi.addActionListener(
	    new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		if (fSimulation.isSimulationActive()) {
		  showStatus("Terminate the current simulation "+
		    "before a "+name+".");
		} else {
		  try {
		    fSimulation.performLint(lintNr);
		    showStatus(upperName+" successful.");
                    closeSyntaxErrorFrame();
		    return;
		  }
		  catch (SyntaxException e) {
		    processSyntaxException(new FigureException(e),true);
		  }
		}
	      }
	    }
	  );
	  menu.add(mi);
	}
	
        return menu;
    }

    private void closeAllSimulationDrawings() {
       boolean goon;
       do {
         goon=false;
         Enumeration drawenum=drawings();
         while (drawenum.hasMoreElements()) {
                Drawing drawing=(Drawing)drawenum.nextElement();
                if (drawing instanceof CPNInstanceDrawing || drawing instanceof TokenBagDrawing) {
                   closeDrawing(drawing);
                   goon=true;
                   break;
                }
         }
       } while (goon);
    }

    protected StandardDrawingView createDrawingView(int width, int height) {
        return new CPNDrawingView(this, width, height);
    }


    protected Menu createFileMenu() {
	Menu menu = super.createFileMenu();
    	MenuItem mi = new MenuItem("Export PostScript...", new MenuShortcut('e'));
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		promptExportPostscript();
	      }
	   }
	);
	menu.insert(mi,menu.getItemCount()-2);

	mi = new MenuItem("Export to Macao...");
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		promptExportMacao();
	      }
	   }
	);
	menu.insert(mi,menu.getItemCount()-2);

	mi = new MenuItem("Export to Woflan...");
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		promptExportWoflan();
	      }
	   }
	);
	menu.insert(mi,menu.getItemCount()-2);

	mi = new MenuItem("Export All to Shadow Simulator...");
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		promptExportShadow();
	      }
	   }
	);
	menu.insert(mi,menu.getItemCount()-2);

/* no fs functions for now
	mi = new MenuItem("Export All Concepts as XML...");
       	mi.addActionListener(
	   new ActionListener() {
	      public void actionPerformed(ActionEvent event) {
		promptExportConceptsXML();
	      }
	   }
	);
	menu.insert(mi,menu.getItemCount()-2);
*/

        return menu;
    }

    protected Menu createAttributesMenu() {
        Menu menu = super.createAttributesMenu();

        CommandMenu textTypeMenu = new CommandMenu("TextType");
        textTypeMenu.add(new ChangeAttributeCommand("Label",
						    "TextType",
			           new Integer(CPNTextFigure.LABEL), this));
        textTypeMenu.add(new ChangeAttributeCommand("Inscription",
						    "TextType",
			           new Integer(CPNTextFigure.INSCRIPTION), this));
        textTypeMenu.add(new ChangeAttributeCommand("Name",
						    "TextType",
				   new Integer(CPNTextFigure.NAME), this));
	menu.add(textTypeMenu);
	

	menu.addSeparator();
        CommandMenu traceMenu = new CommandMenu("Trace");
        traceMenu.add(new ChangeAttributeCommand("on",
						 "TraceMode",
						 Boolean.TRUE,
						 this));
        traceMenu.add(new ChangeAttributeCommand("off",
						 "TraceMode",
						 Boolean.FALSE,
						 this));
	menu.add(traceMenu);
        return menu;
    }

    protected void drawingAdded(Drawing drawing) {
      super.drawingAdded(drawing);
      if (drawing instanceof CPNDrawing) {
	fSimulation.addDrawing((CPNDrawing)drawing);
      }
    }

    protected void drawingReleased(Drawing drawing) {
      super.drawingReleased(drawing);
      if (drawing instanceof CPNDrawing) {
	fSimulation.releaseDrawing((CPNDrawing)drawing);
      }
    }

    private SyntaxExceptionFrame syntaxFrame=null;

    public void closeSyntaxErrorFrame() {
      if (syntaxFrame!=null) {
          syntaxFrame.dispose();
          syntaxFrame=null;
      }
    }

    void processSyntaxException(FigureException e,
				boolean displayImmediately) {
      if (syntaxFrame==null) {
	syntaxFrame=new SyntaxExceptionFrame(this);
      }

      syntaxFrame.displayException(e,displayImmediately);

      // Print the error message.
      showStatus("A syntax error occurred.");
    }

    boolean selectOffendingElements(FigureException e) {

      if (e.errorDrawing != null && !e.errorFigures.isEmpty()) {
	view().clearSelection();
	showDrawingViewFrame(e.errorDrawing);
	FigureEnumeration errorFigures=new FigureEnumerator(e.errorFigures);
	while (errorFigures.hasMoreElements()) {
	  view().addToSelection(errorFigures.nextFigure());
	}
	
	// Redraw the newly selected elements.	
	view().repairDamage();
	if (e.textErrorFigure != null) {
          doTextEdit(e.textErrorFigure,e.line,e.column);
        }
	return true;
      } else {
	return false;
      }
    }

    public void openInstanceDrawing(NetInstance instance) {
      // Should provide an explicit getName() for Net.
      CPNDrawing drawing=fSimulation.getDrawing(instance.getNet().toString());

      DrawingViewFrame viewFrame=showDrawingViewFrame(
	CPNInstanceDrawing.getInstanceDrawing(instance,drawing,
	  fSimulation.getFigureLookup()));
      viewFrame.onDiscardRelease();
      viewFrame.view().setBackground(INSTANCE_COLOR);
    }

    public void openTokenBagDrawing(PlaceInstance pi) {
      DrawingViewFrame viewFrame=showDrawingViewFrame(
	TokenBagDrawing.getTokenBagDrawing(pi));
      viewFrame.onDiscardRelease();
      viewFrame.view().setBackground(TOKEN_BAG_COLOR);
    }

    /**
     * Initialize the simulation.
     */
    void initSimulation() {
      if (!(drawing() instanceof CPNDrawing)) {
	showStatus("Cannot start new simulation with this drawing! Please activate a net window.");
	return;
      }

      try {
	NetInstance primaryInstance=
	  fSimulation.initSimulation(drawing().getName(),this);

	if (primaryInstance!=null) {
	  openInstanceDrawing(primaryInstance);
	  
	  if (fSimulation.isSimulationActive()) {
	    showStatus("Simulation initialized.");
	  } else {
	    showStatus("Simulation initialized. "+
	      "No enabled bindings.");
	  }
	} else {
	  showStatus("The simulation could not be initialized.");
	}
        //closeSyntaxErrorFrame();
      } catch (SyntaxException e) {
	processSyntaxException(new FigureException(e),true);
      }
    }

    /**
     * Shows a file dialog and saves drawing as postscript.
     */
    public void promptExportPostscript() {
        toolDone();
        String path = getSavePath("Export Postscript...",drawing().getName()+".ps");
        if (path != null) {
            if (!path.endsWith(".ps"))
                path += ".ps";
            exportPostscript(path);
        }
    }

    public void promptExportMacao() {
	if (drawing() instanceof CPNDrawing) {
	  toolDone();
	  String path = getSavePath("Export to Macao...",drawing().getName());
	  if (path != null) {
	    try {
	      PrintWriter p = new PrintWriter(new FileOutputStream(path));
	      ((CPNDrawing)drawing()).exportMacao(p);
	      p.close();
	    } catch (Exception e) {
	      showStatus(e.toString());
	    }
	  }
	} else {
	  showStatus("This drawing is not a net!");
	}
    }

    public void promptExportWoflan() {
	if (drawing() instanceof CPNDrawing) {
	  toolDone();
	  String path = getSavePath("Export to Woflan...",drawing().getName());
	  if (path != null) {
	    fSimulation.buildAllShadows();
	    try {
	      PrintWriter p = new PrintWriter(new FileOutputStream(path));
	      ((CPNDrawing)drawing()).exportWoflan(p);
	      p.close();
	    } catch (Exception e) {
	      showStatus(e.toString());
	    }
	  }
	} else {
	  showStatus("This drawing is not a net!");
	}
    }

/*
    public void promptExportConceptsXML() {
      toolDone();
      String path = getSavePath("Export All Concepts as XML...",null);

      if (path != null) {
	try {
            PrintWriter p = new PrintWriter(new FileOutputStream(path));
            p.println("<?xml version=\"1.0\"?>");
            p.println("<!DOCTYPE FS SYSTEM \"FS.dtd\">");
            p.println("<FS>");
            p.print(fs.ConceptImpl.allToXML());
            p.println("</FS>");
            p.close();
	} catch (Exception e) {
	  showStatus(e.toString());
	}
      }
    }
*/

    public void promptExportShadow() {
      toolDone();
      String path = getSavePath("Export all to Shadow Simulator...",null);

      if (path != null) {
	try {
            FileOutputStream stream = new FileOutputStream(path);
            ObjectOutput output = new ObjectOutputStream(stream);
            output.writeObject(fSimulation.getNetSystem());
            output.close();
	} catch (Exception e) {
	  showStatus(e.toString());
	}
      }
    }

    private void exportPostscript(String file) {
        try {
            PostscriptWriter ps = new PostscriptWriter(file);
	    Dimension pageSize = getDrawingViewSize();
	    Rectangle drawingBox = ((Figure)drawing()).displayBox();
	    ps.translate(32,32+pageSize.height);
	    pageSize.width = Math.max(pageSize.width,drawingBox.x+drawingBox.width);
	    pageSize.height = Math.max(pageSize.height,drawingBox.y+drawingBox.height);
	    //ps.translate(-32,-32+pageSize.height);
	    //ps.setClip(64,64,pageSize.width-128,pageSize.height-128);
	    // replaced by:
	    ps.setClip(0,0,pageSize.width,pageSize.height);
	    double scale=Math.min(535.0/pageSize.width,778.0/pageSize.height);
            if (scale < 1.0) {
	      System.out.println("Scaling PS output by "+scale);
	      ps.scale(scale,scale);
	    }
	    drawing().draw(ps);
            ps.dispose();
        } catch (java.io.IOException e) {
            System.out.println(e.toString());
            showStatus(e.toString());
        }
    }

    protected StorableInput makeStorableInput(FileInputStream stream,
      boolean useUFT) 
    {
      return new PatchingStorableInput(stream,useUFT);
    }

    /**
     * Override to define the dimensions of the drawing view.
     */
    protected Dimension getDrawingViewSize() {
        return new Dimension(535,778);
    }

    /**
     * Gets the default size of the window.
     */
    protected Dimension defaultSize() {
        return new Dimension(640,800);
    }

    public ToolButton toolButtonForTextFigure(TextFigure figure) {
       if (figure instanceof CPNTextFigure) {
          if (figure instanceof DeclarationFigure)
             return fDeclTB;
          switch (((CPNTextFigure)figure).getType()) {
            case CPNTextFigure.INSCRIPTION: return fInscrTB;
            case CPNTextFigure.NAME: return fNameTB;
            default: { }
          }
       }
       return super.toolButtonForTextFigure(figure);
    }

    protected void createAdditionalTools(Panel palette) {
        Tool tool;

	String compilerType=System.getProperty("de.renew.netCompiler");

	palette.add(new java.awt.Canvas());
	palette.add(new java.awt.Canvas());

        tool = new TransitionFigureCreationTool(this);
        palette.add(createToolButton(CPNIMAGES+"TRANS","Transition Tool", tool));

        tool = new PlaceFigureCreationTool(this);
        palette.add(createToolButton(CPNIMAGES+"PLACE","Place Tool", tool));

        tool = new VirtualPlaceFigureCreationTool(this);
        palette.add(createToolButton(CPNIMAGES+"VPLACE","Virtual Place Tool", tool));

        tool = new ConnectionTool(this, ArcConnection.NormalArc);
        palette.add(createToolButton(CPNIMAGES+"ARC", "Arc Tool", tool));

        tool = new ConnectionTool(this, ArcConnection.TestArc);
        palette.add(createToolButton(IMAGES+"LINE", "TestArc Tool", tool));

        tool = new ConnectionTool(this, ArcConnection.ReserveArc);
        palette.add(createToolButton(IMAGES+"CONN", "ReserveArc Tool", tool));

	CPNTextFigure inscription;
	inscription = CPNTextFigure.Inscription;
        tool = new CPNTextTool(this, inscription);
        // store the inscription text tool button
	// for immediate editing of errors:
	fInscrTB=createToolButton(CPNIMAGES+"INSCR","Inscription Tool",tool);
        palette.add(fInscrTB);

        tool = new ConnectedTextTool(this, CPNTextFigure.Name);
        fNameTB = createToolButton(CPNIMAGES+"NAME","Name Tool",tool);
        palette.add(fNameTB);

        tool = new CPNTextTool(this, new DeclarationFigure());
        fDeclTB = createToolButton(CPNIMAGES+"DECL", "Declaration Tool", tool);
        palette.add(fDeclTB);

	if (compilerType!=null && compilerType.indexOf("FS")>=0) {
	  palette.add(new java.awt.Canvas());
	  palette.add(new java.awt.Canvas());
	  
	  tool = new ConnectedTextTool(this, new ConceptFigure());
	  palette.add(createToolButton(IMAGES+"PERT",
	    "Concept Tool", tool));
	  
	  tool = new ConnectionTool(this, new IsaConnection());
	  palette.add(createToolButton(CPNIMAGES+"ISA",
	    "Is-A Tool", tool));
	  
	  tool = new PartitionCreationTool(this);
	  palette.add(createToolButton(IMAGES+"RECT",
	    "Concept Partition Tool", tool));
	}

	palette.add(new java.awt.Canvas());
	palette.add(new java.awt.Canvas());

    }

    /**
     * Creates the selection tool used in this editor.
     */
    protected Tool createSelectionTool() {
        return new CPNSelectionTool(this);
    }

    //-- main -----------------------------------------------------------

    public static void main(String[] args) {
	new CPNApplication(args);
    }
}

class SyntaxExceptionFrame implements FigureChangeListener {
  
  private final Frame frame;
  private final Button selectButton;
  private final TextArea textArea;
  private final CPNApplication editor;
  private FigureException e=null;
  
  SyntaxExceptionFrame(final CPNApplication editor) { 
    this.editor=editor;
    frame=new Frame("Renew: Syntax Error");
    frame.setIconImage(Toolkit.getDefaultToolkit()
      .createImage(editor.menuFrame().getIconImage().getSource()));
    textArea=new TextArea();
    textArea.setEditable(false);
    frame.add("Center",textArea);
    Panel southPanel=new Panel(new GridLayout(1,2));
    frame.add("South",southPanel);
    Button closeButton=new Button("Close");
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
	frame.dispose();
      }
    });
    selectButton=new Button("Select");
    selectButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
	if (e!=null && !editor.selectOffendingElements(e))
	  selectButton.setEnabled(false);
      }
    });
    southPanel.add("1",closeButton);
    southPanel.add("2",selectButton);
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent event) {
	dispose();
      }
    });
  }

  void displayException(FigureException e, boolean displayImmediately) {
    if (displayImmediately)
      selectButton.setEnabled(editor.selectOffendingElements(e));
    else
      selectButton.setEnabled(true);
    textArea.setText(e.getMessage());
    this.e=e;

    FigureEnumeration errorFigures=new FigureEnumerator(e.errorFigures);
    while (errorFigures.hasMoreElements()) {
        errorFigures.nextFigure().addFigureChangeListener(this);
    }

    frame.pack();
    frame.setVisible(true);
  }

    public void figureInvalidated(FigureChangeEvent e) { }

    public void figureRequestRemove(FigureChangeEvent e) { }

    public void figureRequestUpdate(FigureChangeEvent e) { }

    public void figureChanged(FigureChangeEvent e) { }

    public void figureRemoved(FigureChangeEvent e) {
       dispose();
    }

    public void dispose() {
      if (e!=null) {
        frame.dispose();
        FigureEnumeration errorFigures=new FigureEnumerator(e.errorFigures);
        while (errorFigures.hasMoreElements()) {
            errorFigures.nextFigure().removeFigureChangeListener(this);
        }
        e=null;
      }
    }

}


