package de.renew.gui;

import java.util.*;
import java.awt.*;
import java.io.IOException;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.StorableInput;
import de.renew.shadow.*;
import de.renew.simulator.Arc;

public class CPNDrawing extends StandardDrawing {
  private ShadowNet shadowNet=null;


  public CPNDrawing() {
    super();
  }

  public void release() {
    super.release();
    discardShadow();
  }

  static CPNDrawing findDrawing(Object errorObject) {
    // Determine the drawing containing the errorObject.
    if (errorObject instanceof ShadowNetElement) {
       return (CPNDrawing)((ShadowNetElement)errorObject).getNet().context;
    }
    return null;
  }

  private String macaoString(String str) {
    return String.valueOf(str.length())+":"+str;
  }

  private boolean exportTextChildren(java.io.PrintWriter output,
				     ParentFigure figure, int node,
				     String inscriptionType) {
    boolean isInit = false;
    // Look for text children:
    FigureEnumeration fe = figure.children();
    while (fe.hasMoreElements()) {
      Figure child = fe.nextFigure();
      if (child instanceof CPNTextFigure) {
	CPNTextFigure textchild = (CPNTextFigure)child;
	String name = textchild.getText();
	String texttype = null;
	int textType = textchild.getType();
	if (textType == CPNTextFigure.NAME)
	  texttype = "name";
	else if (textType == CPNTextFigure.INSCRIPTION) {
	  if (name.startsWith(":new("))
	    isInit = true;
	  texttype = inscriptionType;
	  inscriptionType = null; // only 1 inscription allowed!
	}
	if (texttype != null) {
	  if (name.length() > 0) {
	    if (texttype.equals("valuation"))
	      name = "<"+name+">";
	    output.println("CT("+macaoString(texttype)+","+node
			   +","+macaoString(name)+")");
	    Point origin = textchild.getOrigin();
	    output.println("PT("+node+","+macaoString(texttype)+","
			   +origin.x+","+origin.y+")");
	  }
	}
      }
    }
    return isInit;
  }

  private void exportArc(java.io.PrintWriter output,
			 ArcConnection arc,
			 Hashtable table,
			 Integer start, Integer end,
			 int node, boolean reverse) {
    output.println("CA("+macaoString("arc")+"," + (++node) + ","
                   + start + "," + end + ")");
    table.put(arc,new Integer(node));
    int lastpoint = arc.pointCount()-1;
    for (int i=1; i < lastpoint; ++i) {
      Point p = arc.pointAt(reverse ? lastpoint-i : i);
      output.println("PI(-1,"+node+","+(p.x-3)+","+(p.y-3)+",-1)");
    }
    exportTextChildren(output,arc,node,"valuation");
    table.put(arc,new Integer(node));
  }

  public void exportMacao(java.io.PrintWriter output) {
    output.println("DB()");
    int node = 1; // node 1 is reserved for the declaration node!
    // create initially marked place as node 2:
    output.println("CN("+macaoString("place")+","+(++node)+")");
    output.println("CT("+macaoString("marking")+","+node+","
      +macaoString("1")+")");

    Hashtable table = new Hashtable();
    FigureEnumeration figenum = figures();
    while (figenum.hasMoreElements()) {
      Figure figure = figenum.nextFigure();
      if (figure instanceof NodeFigure) {
	String type, inscrType;
	if (figure instanceof PlaceFigure) {
	  type = "place";
	  inscrType = "domain";
	} else {
	  type = "transition";
	  inscrType = null; // "guard";
	}
	output.println("CN("+macaoString(type)+"," + (++node) + ")");
	Rectangle box = figure.displayBox();
	output.println("PO(-1,"+node+","+(box.x)+","+(box.y)+","
		       +(box.x+box.width)+","+(box.y+box.height)+")");
        table.put(figure,new Integer(node));
	if (exportTextChildren(output,(ParentFigure)figure,node,inscrType)) {
	  Point center = figure.center();
	  output.println("PO(2,"+center.x+","+(center.y-3*box.height)+")");
	  output.println("CA("+macaoString("arc")+","+(node+1)+",2,"+node+")");
	  ++node;
	}
      }
    }
    figenum = figures();
    while (figenum.hasMoreElements()) {
      Figure figure = figenum.nextFigure();
      if (figure instanceof ArcConnection) {
	ArcConnection arc = (ArcConnection)figure;
	Integer start = (Integer)table.get(arc.startFigure());
	Integer end = (Integer)table.get(arc.endFigure());
	boolean twoarcs = arc.getArcType() != ShadowArc.ordinary;
	boolean reverse = arc.isReverse();
	if (twoarcs || !reverse)
	  exportArc(output,arc,table,start,end,++node,false);
	if (twoarcs || reverse)
	  exportArc(output,arc,table,end,start,++node,true);
      }
    }
    output.println("FB()");
  }


  // It is assumed that my shadow objects are up to date.
  public void exportWoflan(java.io.PrintWriter output) {
    output.println("place pinit init 1;");
    int cnt = 0;
    FigureEnumeration placeenum = figures();
    while (placeenum.hasMoreElements()) {
      Figure figure = placeenum.nextFigure();
      if (figure instanceof PlaceFigure) {
	ShadowPlace place = (ShadowPlace)((ShadowHolder)figure).getShadow();
	String name = place.getName();
	if (name == null) {
	  name = "P"+(++cnt);
	  place.setName(name);
	}
	output.println("place "+name+";");
      }
    }
    cnt = 0;
    FigureEnumeration transenum = figures();
    while (transenum.hasMoreElements()) {
      Figure figure = transenum.nextFigure();
      if (figure instanceof TransitionFigure) {
	ShadowTransition trans = (ShadowTransition)((ShadowHolder)figure).getShadow();
	String name = trans.getName();
	if (name == null) {
	  name = "T"+(++cnt);
	  trans.setName(name);
	}
	Vector inarcs = new Vector();  // ...of Placenames
	Vector outarcs = new Vector(); // dito
	// test if transition is :init(...)-transition:
	Enumeration inscrenum = trans.elements();
	while (inscrenum.hasMoreElements()) {
	  Object elem = inscrenum.nextElement();
	  if (elem instanceof ShadowInscription) {
	    if (((ShadowInscription)elem).inscr.startsWith(":new(")) {
	      inarcs.addElement("pinit");
	    }
	  }
	}
	FigureEnumeration arcenum = figures();
	while (arcenum.hasMoreElements()) {
	  figure = arcenum.nextFigure();
	  if (figure instanceof ArcConnection) {
	    ArcConnection ac = (ArcConnection)figure;
	    ShadowArc arc = (ShadowArc)ac.getShadow();
	    if (arc.transition == trans) {
	      String placename = arc.place.getName();
	      if (arc.arcType == Arc.both || arc.arcType == Arc.test
		  || arc.arcType == Arc.in)
		inarcs.addElement(placename);
	      if (arc.arcType == Arc.both || arc.arcType == Arc.test
		   || arc.arcType == Arc.out)
		outarcs.addElement(placename);
	    }
	  }
	}
	output.println("trans "+name+placeVector(inarcs," in")+placeVector(outarcs," out")+";");
      }
    }
  }

  private String placeVector(Vector places, String title) {
    StringBuffer output = new StringBuffer("");
    if (places.size() > 0) {
      output.append(title);
      for (int i=places.size()-1; i>=0; --i)
	output.append(" ").append(places.elementAt(i));
    }
    return output.toString();
  }

  /**
   * Draws all figures but the initial markings
   * @see Figure#draw
   */
  public void drawWithoutInitialMarkings(Graphics g) {
    FigureEnumeration k = figures();
    while (k.hasMoreElements()) {
      Figure fig=k.nextFigure();
      if (!(fig instanceof CPNTextFigure
	&& ((CPNTextFigure)fig).getType() == CPNTextFigure.INSCRIPTION
	&& ((CPNTextFigure)fig).parent() instanceof PlaceFigure)) {
	fig.draw(g);
      }
    }
  }
  
  
  public void discardShadow() {
     if (shadowNet != null) {
        shadowNet.discard();
        shadowNet=null;
     }
  }


  private void buildShadow(Class clazz, ShadowNet shadowNet) {
        FigureEnumeration k = figures();
        while (k.hasMoreElements()) {
           Figure fig=k.nextFigure();
           if (clazz.isInstance(fig)) {
              ((ShadowHolder)fig).buildShadow(shadowNet);
           }
        }
  }

  public ShadowNet buildShadow(ShadowNetSystem netSystem) {
        discardShadow();
        shadowNet = new ShadowNet(getName(),netSystem);
        shadowNet.context = this;

        // Build new shadows of all elements.
        // Do everything in a fine order:
        // First declaration nodes, then nodes, then arcs, then inscriptions.

        // Build shadows for declaration nodes (java-nets: only one!)
        buildShadow(DeclarationFigure.class,shadowNet);

        // Build shadows for nodes (transitions and places):
        buildShadow(NodeFigure.class,shadowNet);

        // Build shadows for arcs:
        buildShadow(ArcConnection.class,shadowNet);

        // Build shadows for inscriptions:
        buildShadow(CPNTextFigure.class,shadowNet);

        return shadowNet;
  }
}

