package de.renew.gui;

import java.util.*;
import java.awt.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.framework.*;
import de.renew.simulator.*;

public class CPNInstanceDrawing extends StandardDrawing
                                implements DrawingChangeListener {

  private static Hashtable drawingsByInstance = new Hashtable();

  private CPNInstanceDrawing(NetInstance netInstance,
    CPNDrawing cpnDrawing, FigureLookup figureLookup)
  {
    this.cpnDrawing = cpnDrawing;

    connect(netInstance,figureLookup);
    cpnDrawing.addDrawingChangeListener(this);
  }

  public Rectangle displayBox() {
    return super.displayBox().union(cpnDrawing.displayBox());
  }

  private NetInstance netInstance;
  private CPNDrawing cpnDrawing;

  static CPNInstanceDrawing getInstanceDrawing(NetInstance netInstance,
    CPNDrawing cpnDrawing, FigureLookup figureLookup)
  {
    // forbid concurrent searches for instance drawings.
    synchronized (drawingsByInstance) {
      // find instance drawing:
      if (drawingsByInstance.containsKey(netInstance))
	return (CPNInstanceDrawing)drawingsByInstance.get(netInstance);
      else
	return new CPNInstanceDrawing(netInstance,cpnDrawing,figureLookup);
    }
  }

  public void draw(Graphics g) {
      //g.setColor(Color.yellow.brighter());
      //Rectangle b=displayBox();
      //g.fillRect(b.x,b.y,b.width,b.height);
    cpnDrawing.drawWithoutInitialMarkings(g);
    super.draw(g);
  }

  public void connect(NetInstance netInstance, FigureLookup figureLookup) {
    if (figureLookup != null && netInstance != null) {
      if (this.netInstance != null)
	drawingsByInstance.remove(this.netInstance);
      this.netInstance = netInstance;
      drawingsByInstance.put(netInstance,this);
      setName(netInstance.toString());
      FigureEnumeration figures = cpnDrawing.figures();
      while (figures.hasMoreElements()) {
	Figure figure = figures.nextFigure();
	if (figure instanceof PlaceFigure) {
	  PlaceFigure placeFig = (PlaceFigure)figure;
	  Place place;
          if (placeFig instanceof VirtualPlaceFigure) {
             place=figureLookup.get( ((VirtualPlaceFigure)placeFig).getPlace() );
          } else {
             place=figureLookup.get(placeFig);
          }
	  if (place!=null) {
            add(new ExpandedTokenBagFigure(placeFig,netInstance.getInstance(place)));
	  }
	} else if (figure instanceof TransitionFigure) {
	  TransitionFigure transitionFig = (TransitionFigure)figure;
	  Transition transition=figureLookup.get(transitionFig);
	  if (transition!=null) {
            add(new TransitionInstanceFigure(transitionFig,netInstance.getInstance(transition)));
	  }
	}
      }
    }
  }

  public void release() {
    cpnDrawing.removeDrawingChangeListener(this);
    if (netInstance != null) {
      drawingsByInstance.remove(netInstance);
      //System.err.println("Instance Drawing removed from list.");
      netInstance = null;
    }
    super.release();
  }

  /**
   *  Sent when an area is invalid
   */
  public void drawingInvalidated(DrawingChangeEvent e) {
    // System.out.println("Source invalidated: "+e.getInvalidatedRectangle());
    DrawingChangeEvent myEvent = new DrawingChangeEvent(this,
      e.getInvalidatedRectangle());
    Enumeration listeners = drawingChangeListeners();
    while (listeners.hasMoreElements()) {
      DrawingChangeListener listener = (DrawingChangeListener)listeners.nextElement();
      listener.drawingInvalidated(myEvent);
      listener.drawingRequestUpdate(myEvent);
    }
  }

  /**
   *  Sent when the drawing wants to be refreshed
   */
  public void drawingRequestUpdate(DrawingChangeEvent e) {
    //System.out.println("Source requests update: "+e.getInvalidatedRectangle());
  }


   /**
     * Returns whether drawing has been modified since last save.
     */
    public boolean isModified() {
       return false;  // an InstanceDrawing should not be saved!
    }

}
