package de.renew.gui;

import java.awt.*;
import java.util.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import de.renew.simulator.*;
import collections.*;

public class ExpandedTokenBagFigure extends SimpleCompositeFigure
implements ChildFigure, NetObserver {
    public ExpandedTokenBagFigure() {
        super();
    }

    private static final int BORDER = 3;
    private static final int SEPARATOR = 1;
    private OffsetLocator fLocator = null;
    private PlaceFigure place = null;
    private PlaceInstance placeInstance = null;
    private int cacheMult = -1;
    private int cacheTestMult = -1;
    private CardBagFigure cardFigure;
    private boolean expanded=false;

    public ExpandedTokenBagFigure(PlaceInstance pi) {
       placeInstance = pi;
       cardFigure = new CardBagFigure(pi);
       add(cardFigure);
       update(pi, null);
    }

    public ExpandedTokenBagFigure(PlaceFigure place, PlaceInstance pi) {
       this(pi);
       setParent(place);
    }

    public void draw(Graphics g) {
       if (expanded) {
         g.setColor(Color.red);
         Rectangle box=displayBox();
         g.drawRect(box.x,box.y,box.width-1,box.height-1);
       }
       super.draw(g);
    }

    public boolean canBeParent(ParentFigure p) {
       return p instanceof PlaceFigure;
    }

    private void unsetParent() {
      if (place != null) {
	place.removeChild(this);
	place=null;
      }
    }

    public boolean setParent(ParentFigure figure) {
        if (!canBeParent(figure))
	  return false;
	if (place == figure)
	  return true;
	unsetParent();
        place = (PlaceFigure)figure;
	if (place == null) {
          fLocator = null;
        } else {
	  place.addChild(this);
          if (fLocator != null)
             fLocator.setBase(place.connectedTextLocator(this));
          else
	     fLocator = new OffsetLocator(place.connectedTextLocator(this));
	  updateLocation();
	}
	return true;
    }

    public ParentFigure parent() { return place; }

    protected void layout() {
       Point partOrigin = displayBox().getLocation();
       partOrigin.translate(BORDER,BORDER);
       Dimension extent = new Dimension(0, 0);

       FigureEnumeration k = figures();
       while (k.hasMoreElements()) {
            Figure f = k.nextFigure();

 	    Dimension partExtent = f.size();
	    Point corner = new Point(
		                        partOrigin.x+partExtent.width,
		                        partOrigin.y+partExtent.height);
            f.basicDisplayBox(partOrigin, corner);

	    extent.width = Math.max(extent.width, partExtent.width);
	    extent.height += partExtent.height+SEPARATOR;
	    partOrigin.y += partExtent.height+SEPARATOR;
        }
        fDisplayBox.width = extent.width + 2*BORDER;
        fDisplayBox.height = extent.height - SEPARATOR + 2*BORDER;
        updateLocation();
    }

    protected boolean needsLayout() {
        Dimension extent = new Dimension(0, -SEPARATOR + 2*BORDER);

        FigureEnumeration k = figures();
        while (k.hasMoreElements()) {
            Figure f = k.nextFigure();
            extent.width = Math.max(extent.width, f.size().width);
            extent.height += f.size().height + SEPARATOR;
        }
        extent.width += 2*BORDER;
        return !extent.equals(fDisplayBox.getSize());
    }

    public void moveBy(int x, int y) {
        super.moveBy(x, y);
        if (fLocator != null)
            fLocator.moveBy(x, y);
    }

    /**
     * Updates the location relative to the connected figure.
     * The Figure is centered around the located point.
     */
    public void updateLocation() {
        if (fLocator != null) {
            Point p = fLocator.locate(place);
            Rectangle box=displayBox();
            p.x -= box.x + box.width/2;
            p.y -= box.y + box.height/2;

            if (p.x != 0 || p.y != 0) {
                willChange();
                basicMoveBy(p.x, p.y);
                changed();
            }
        }
    }


  public void update(Object oPlaceInstance, Object freeTokens) {
    // Updates only originate from place instances.
    // This is strange: Why should the attached place instance
    // change?
    if (placeInstance != oPlaceInstance) {
      placeInstance.deleteObserver(this);
      placeInstance = (PlaceInstance)oPlaceInstance;
      // construct new cardFigure!
    }

    if (!expanded) {
      cardFigure.update();
    } else {
      willChange();

      placeInstance.lock.lock();

      TokenBag myBag=placeInstance.getTokens();
      removeAll();
      Enumeration tokens = placeInstance.getDistinctTestableTokens();
      while (tokens.hasMoreElements()) {
         Object token=tokens.nextElement();
         int mult=myBag.getMultiplicity(token);
         boolean isTested=placeInstance.containsTestedToken(token);
         add(new TokenFigure(mult,isTested,token));
      }

      placeInstance.lock.unlock();

      layout();
      changed();

      if (listener() != null)
	listener().figureRequestUpdate(new FigureChangeEvent(this));
    }

    placeInstance.addObserver(this);

  }

  public void release() {
    if (placeInstance != null) {
      placeInstance.deleteObserver(this);
      // OK: Release reference to place instance so that the place
      // instance can be garbage collected.
      placeInstance=null;
    }
    // OK: Forget about the place.
    unsetParent();
    // OK: Forget about all tokens.
    removeAll();

    super.release();
  }

  public String toString() {
    if (placeInstance==null) {
      return "Detached ExpandedTokenBagFigure@"+System.identityHashCode(this);
    }

    StringBuffer output = new StringBuffer().append('{');

    placeInstance.lock.lock();

    TokenBag myBag=placeInstance.getTokens();
    Enumeration tokens = placeInstance.getDistinctTestableTokens();
    while (tokens.hasMoreElements()) {
      Object token=tokens.nextElement();
      int mult=myBag.getMultiplicity(token);
      boolean isTested=placeInstance.containsTestedToken(token);

      output.append(TokenFigure.getMultString(mult,isTested));
      output.append(token.toString());

      if (tokens.hasMoreElements())
	output.append(", ");
    }

    placeInstance.lock.unlock();

    output.append('}');
    return output.toString();
  }

  public void toggleExpanded() {
       willChange();
       expanded = !expanded;
       if (!expanded) {
          removeAll();
          add(cardFigure);
       }
       update(placeInstance, null);
       layout();
  }

  public boolean inspect(DrawingView view, boolean alternate) {
     if (alternate) {
      if (!super.inspect(view,true)) {
         toggleExpanded();
      }
      return true;
     } else {
      return super.inspect(view,alternate);
     }
  }

  protected PlaceInstance getPlaceInstance() {
     return placeInstance;
  }

}

