package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import collections.CollectionEnumeration;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import de.renew.simulator.*;

class TransitionInstanceFigure extends AbstractFigure {

  private TransitionFigure transitionFigure;
  private TransitionInstance transitionInstance;

  public TransitionInstanceFigure(TransitionFigure tf, TransitionInstance ti) {
        transitionInstance = ti;
        transitionFigure = tf;
  }

    public void basicDisplayBox(Point origin, Point corner) {
        // do nothing, as the DisplayBox of the Transition is used!
    }

    public Vector handles() {
        Vector handles = new Vector();
        handles.addElement(new NullHandle(this, RelativeLocator.northWest()));
        handles.addElement(new NullHandle(this, RelativeLocator.northEast()));
        handles.addElement(new NullHandle(this, RelativeLocator.southWest()));
        handles.addElement(new NullHandle(this, RelativeLocator.southEast()));
        return handles;
    }

    public Rectangle displayBox() {
        return transitionFigure.displayBox();
    }

    protected void basicMoveBy(int x, int y) {
        // do nothing, as the DisplayBox of the Transition is used!
    }

    public void draw(Graphics g) {
/*        g.setColor(Color.red);
        Rectangle r = displayBox();
        g.drawRect(r.x, r.y, r.width-1, r.height-1);
*/
    }


  public boolean inspect(DrawingView view, boolean alternate) {
     //System.out.println("Searching bindings for "+transitionInstance+"...");
     if (alternate) {
       if (!fire()) {
          view.editor().showStatus("No binding for this transition.");
       }
     } else {
        BindingSelectionFrame.open(transitionInstance);
     }
     return true;
  }

  public boolean fire() {
    return SimulatorHelper.fireOneBinding(transitionInstance, new ConsoleTracer("Synchronously","(Automatically) "), true);
  }

  public void release() {
    if (transitionInstance != null) {
      // OK: Release reference to transition instance so that it
      // can be garbage collected.
      transitionInstance=null;
    }
    super.release();
  }

  static public void closeBindingSelectionFrame() {
     BindingSelectionFrame.close();
  }
}

class BindingSelectionFrame implements ActionListener {
  private static BindingSelectionFrame instance=null;
   
  static void open(TransitionInstance transitionInstance) {
    if (instance==null) {
      instance=new BindingSelectionFrame();
    }
    instance.setTransitionInstance(transitionInstance);
  }

  static void close() {
     if (instance != null) {
       instance.frame.setVisible(false);
       instance.bindingList.removeAll();
       instance.transitionInstance=null;
     }
  }
  
  private final Frame frame;
  private final List bindingList;
  private final Button fireButton;
  private Binding[] binding;
  private TransitionInstance transitionInstance;
  
  private BindingSelectionFrame() {
    frame=new Frame();

    bindingList=new List();
    bindingList.addActionListener(this);
    frame.add(bindingList,"Center");

    fireButton = new Button(" Fire ");
    fireButton.addActionListener(this);

    Button updateButton = new Button(" Update ");
    updateButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	update();
      }
    });

    Button closeButton = new Button(" Close ");
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         close();
      }
    });

    Panel buttonPanel = new Panel();
    buttonPanel.add(fireButton);
    buttonPanel.add(updateButton);
    buttonPanel.add(closeButton);
    frame.add(buttonPanel,"South");
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	frame.setVisible(false);
      }
    });
    frame.pack();
  }

  public void actionPerformed(ActionEvent e) {
    int index=bindingList.getSelectedIndex();
    if (index>=0 && index<binding.length) {
      //frame.setVisible(false);
      binding[index].execute(new ConsoleTracer("Synchronously","(Manually) "),true);
      update();
    } else {
      fireButton.setEnabled(false);
    }
  }

  private void setTransitionInstance(TransitionInstance transitionInstance) {
    this.transitionInstance=transitionInstance;
    frame.setTitle(transitionInstance+"'s possible bindings");
    update();
  }

  private void update() {
    CollectionEnumeration bindings=SimulatorHelper.findAllBindings(transitionInstance);
    final int size=bindings.numberOfRemainingElements();
    binding = new Binding[size];
    // copy Bindings to Array and rebuild the AWT list.
    bindingList.removeAll();
    for (int i=0; bindings.hasMoreElements(); ++i) {
      binding[i] = (Binding)bindings.nextElement();
      bindingList.addItem(binding[i].toString());
    }
    if (size>0) {
      bindingList.select(0);
    }

    fireButton.setEnabled(size != 0);
    frame.setVisible(true);
  }
}
