/*
 * @(#)VirtualPlaceFigure.java 5.1
 *
 */

package de.renew.gui;

import java.awt.*;
import java.util.*;
import java.io.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import de.renew.simulator.*;
import de.renew.shadow.*;

public class VirtualPlaceFigure extends PlaceFigure {
    /*
     * Serialization support.
     */
    private static final long serialVersionUID = -7877776241236946511L;
    private int virtualPlaceFigureSerializedDataVersion = 1;

    private PlaceFigure place=null;

    public VirtualPlaceFigure() {      // No-Arg-Constructor for loading only!
        super();
    }

    public VirtualPlaceFigure(PlaceFigure place) {
        super();
        setPlace(place);
    }

    private void setPlace(PlaceFigure place) {
        this.place=place;
        adapt();
        final FigureChangeEvent removeEvent = new FigureChangeEvent(this, null);
        place.addFigureChangeListener(new FigureChangeAdapter() {
           public void figureChanged(FigureChangeEvent e) {
              adapt();
           }

           public void figureRemoved(FigureChangeEvent e) {
              if (listener()!=null) {
                 listener().figureRequestRemove(removeEvent);
              }
           }

        });
    }

  public PlaceFigure getPlace() {
     return place;
  }

  private void adapt() {
        super.setAttribute("FillColor",place.getFillColor());
        super.setAttribute("FrameColor",place.getFrameColor());
  }

  public void drawFrame(Graphics g) {
     super.drawFrame(g);
     Rectangle r = displayBox();
     g.drawOval(r.x+2, r.y+2, r.width-5, r.height-5);
  }

  public ShadowNetElement buildShadow(ShadowNet net) {
      return null;
  }

  public ShadowNetElement getShadow() {
      return place.getShadow();
  }

  public void setAttribute(String attr, Object value) {
     if ("FillColor".equals(attr) || "FrameColor".equals(attr)) {
	 place.setAttribute(attr,value);
     } else {
        super.setAttribute(attr,value);
     }
  }

  public boolean inspect(DrawingView view, boolean alternate) {
     if (alternate) {
        return super.inspect(view,true);
     } else {
       view.clearSelection();
       view.addToSelection(place);
       return true;
     }
  }


   /**
     * Stores the Figure to a StorableOutput.
     */
    public void write(StorableOutput dw) {
        super.write(dw);
        dw.writeStorable(place);
    }

    /**
     * Reads the Figure from a StorableInput.
     */
    public void read(StorableInput dr) throws IOException {
        super.read(dr);
        setPlace((PlaceFigure)dr.readStorable());
    }


}

