package de.renew.shadow;

import collections.*;
import de.renew.simulator.Arc;
 
public class ShadowArc extends ShadowInscribable {
  static final long serialVersionUID = -774253458615347237L;

  public static final int test=0;
  public static final int ordinary=1;
  public static final int both=2;
  
  public ShadowTransition transition;
  public ShadowPlace place;

  private boolean trace;
  // The arc type is determined by the constants
  // in class de.renew.simulator.Arc.
  public int arcType;

  private static ShadowNet ensureIdentity(ShadowNode from, ShadowNode to) {
    ShadowNet net=from.getNet();
    if (net!=to.getNet()) {
      throw new RuntimeException("Must connect within one net.");
    }
    return net;
  }
  
  public ShadowArc(ShadowNode from, ShadowNode to,
    int shadowArcType)
  {
    super(ensureIdentity(from,to));

    if (from instanceof ShadowTransition) {
      if (to instanceof ShadowPlace) {
	transition=(ShadowTransition)from;
	place=(ShadowPlace)to;
	arcType=Arc.out;
      } else {
	throw new RuntimeException("Must connect place and transition.");
      }
    } else if (from instanceof ShadowPlace) {
      if (to instanceof ShadowTransition) {
	transition=(ShadowTransition)to;
	place=(ShadowPlace)from;
	arcType=Arc.in;
      } else {
	throw new RuntimeException("Must connect place and transition.");
      }
    } else {
      throw new RuntimeException("Must connect place and transition.");
    }

    // Check the shadow arc type.
    switch (shadowArcType) {
    case test:
      arcType=Arc.test;
      break;
    case both:
      arcType=Arc.both;
      break;
    case ordinary:
      // The arc type has already been determined.
      break;
    default:
      throw new RuntimeException("Bad shadow arc type: "+shadowArcType+".");
    }

    trace=true;

    // Ok, remember the inscription.
    transition.add(this);
  }

  public ShadowArc(ShadowNode from, ShadowNode to) {
    this(from,to,ordinary);
  }

  // Convenience constructor for arcs with inscription.
  public ShadowArc(ShadowNode from, ShadowNode to,
    int shadowArcType, String inscription)
  {
    this(from,to,shadowArcType);
    new ShadowInscription(this,inscription);
  }

  // Convenience constructor for ordinary arcs with inscription.
  public ShadowArc(ShadowNode from, ShadowNode to,
    String inscription) throws SyntaxException 
  {
    this(from,to,ordinary,inscription);
  }

  public void setTrace(boolean trace) {
    if (this.trace!=trace) {
      this.trace=trace;
    }
  }

  public boolean getTrace() {
    return trace;
  }

  public void discard() {
    transition.remove(this);
  }
}
