package de.renew.shadow;

import collections.*;
import java.util.Hashtable;

public class ShadowNetSystem implements java.io.Serializable {
  static final long serialVersionUID = 4619730346099695518L;

  UpdatableSet nets;
  ShadowCompiler compiler=null;

  public ShadowNetSystem() {
    nets=new HashedSet();
  }

  public ShadowNetSystem(ShadowCompiler compiler) {
    this();
    setCompiler(compiler);
  }

  public void setCompiler(ShadowCompiler compiler) {
     this.compiler=compiler;
  }

  public ShadowCompiler getCompiler() {
     return compiler;
  }

  // Add and remove nets. These methods should be called
  // indirectly via ShadowNet() and ShadowNet.discard(), so that
  // inconsistencies in the data structure cannot arise.
  protected void add(ShadowNet net) {
    nets.include(net);
  }
  protected void remove(ShadowNet net) {
    nets.exclude(net);
  }

  public ShadowLookup compile() throws SyntaxException {
    if (compiler != null) {
      return compiler.compile(this);
    } else {
      return null;
    }
  }

  public void lint(int lintNr) throws SyntaxException {
    if (compiler != null) {
      compiler.lint(lintNr,this);
    }
  }

  public CollectionEnumeration elements() {
    return nets.elements();
  }
}
