package de.renew.simulator;

import de.renew.unify.*;

public class CallExpression implements Expression {
  // Multiple arguments are modelled by an expression that returns
  // a tuple that is processed by a function.
  Expression argument;
  Function function;

  public CallExpression(Expression argument, Function function) {
    this.argument=argument;
    this.function=function;
  }

  public boolean isInvertible() {
    return false;
  }

  public Object startEvaluation(VariableMapper mapper, StateRecorder recorder, 
    CalculationChecker checker) throws Impossible
  {
    final Variable source=
      new Variable(argument.startEvaluation(mapper,recorder,checker),recorder);
    final Variable target=
      new Variable();

    if (checker!=null) {
      checker.addEarlyVariable(source,recorder);
    }

    Unify.expressionConstraint(target,function,source,recorder);

    return target.getValue();
  }

  public Object registerCalculation(VariableMapper mapper, 
    StateRecorder recorder, CalculationChecker checker) throws Impossible
  {
    Variable source=new Variable(argument.registerCalculation(mapper,
      recorder,checker),recorder);
    Variable target=new Variable();

    checker.addLateVariable(source,recorder);
    checker.addCalculated(target,source.getValue(),recorder);

    return target.getValue();
  }
}
