package de.renew.simulator;

import collections.*;
import de.renew.function.Value;
import de.renew.util.*;
import de.renew.unify.*;

// A conditional occurrence is its own binder. This simplifies
// the code a bit.

class ConditionalOccurrence implements ExecutableOccurrence, Binder {
  ConditionalInscription conditionalInscription;

  VariableMapper mapper;
  NetInstance netInstance;

  Variable conditionVariable;

  ConditionalOccurrence(ConditionalInscription conditionalInscription,
    VariableMapper mapper, NetInstance netInstance)
  {
    this.conditionalInscription=conditionalInscription;
    this.mapper=mapper;
    this.netInstance=netInstance;
  }
  
  public CollectionEnumeration makeBinders(Searcher searcher)
    throws Impossible
  {
    conditionVariable=new Variable(
      conditionalInscription.conditionExpression.startEvaluation(mapper,
	searcher.recorder,searcher.calcChecker),
	searcher.recorder);

    // I am my own binder.
    return new SingletonEnumeration(this);
  }

  public int bindingBadness(Searcher searcher) {
    if (Unify.isBound(conditionVariable)) {
      return 1;
    } else {
      return BindingBadness.max;
    }
  }

  private InscriptionOccurrence secondaryOccurrence=null;
  private boolean wantToOccur;
  
  public void bind(Searcher searcher) {
    Object obj=conditionVariable.getValue();
    if (!(obj instanceof Value)) {
      // Expression is badly bound.
      return;
    }

    Object value=((Value)obj).value;
    if (!(obj instanceof Boolean)) {
      // Expression is badly bound.
      return;
    }

    UpdatableSeq binders=new LinkedList();

    int checkpoint=searcher.recorder.checkpoint();

    try {       
      boolean wantToOccur=((Boolean)((Value)obj).value).booleanValue();
      
      if (wantToOccur) {
	secondaryOccurrence=
	  conditionalInscription.inscription.makeOccurrence(mapper,
	    netInstance);
	binders.prependElements(secondaryOccurrence.makeBinders(searcher));
	// After this statement, no further exceptions should be thrown.
	searcher.addBinders(binders.elements());
      } else {
	secondaryOccurrence=null;
      }
      
      searcher.removeBinder(this);
      searcher.search();
      searcher.addBinder(this);
      
      if (wantToOccur) {
	searcher.removeBinders(binders.elements());
      }
    }      
    catch (Impossible e) {
      // This is expected. The secondary occurrence could
      // not create its binders.
    }
    
    searcher.recorder.restore(checkpoint);
  }

  public CollectionEnumeration makeExecutables(Copier copier,
    VariableMapper copiedMapper)
  {
    if (secondaryOccurrence instanceof ExecutableOccurrence) {
      // We also know now that secondaryOccurrence!=null
      return ((ExecutableOccurrence)secondaryOccurrence).makeExecutables(
        copier,copiedMapper);
    } else {
      return new EmptyEnumeration();
    }
  }
}
