package de.renew.simulator;

import de.renew.util.EmptyEnumeration;
import collections.*;

/**
 * A deterministic search queue simply provides searchable
 * objects in the same order in which they arrived at the queue.
 * This is useful for debugging purposes.
 *
 * @author Olaf Kummer
 **/

class DeterministicSearchQueue implements SearchQueueData {
  private UpdatableSeq uncheckedSearchables;
    // UpdatableSeq<Searchable>
  private UpdatableSet isUnchecked=new HashedSet();
    // UpdatableSet<Searchable>

  DeterministicSearchQueue() {
    init(new EmptyEnumeration());
  }

  public void include(Searchable searchable) {
    if (!isUnchecked.includes(searchable)) {
      uncheckedSearchables.insertLast(searchable);
      isUnchecked.include(searchable);
    }
  }

  public boolean isEmpty() {
    return uncheckedSearchables.isEmpty();
  }

  public Searchable extract() {
    Searchable searchable=
      (Searchable)uncheckedSearchables.first();
    
    // Drop this searchable from the list of searchables. It will be
    // checked soon, so it will either turn out
    // to be not enabled, or we must move it to
    // the end of the list to get fairness.
    uncheckedSearchables.exclude(searchable);
    isUnchecked.exclude(searchable);
    
    return searchable;
  }

  public CollectionEnumeration elements() {
    return uncheckedSearchables.elements();
  }

  public void init(CollectionEnumeration enum) {
    uncheckedSearchables=new CircularList();
    isUnchecked=new HashedSet();
    // Insert the elements into the list.
    uncheckedSearchables.appendElements(enum);
    // Insert the elements into the set. Note that
    // the enumeration must not be used twice.
    isUnchecked.includeElements(uncheckedSearchables.elements());
  }
}

