package de.renew.simulator;

import collections.*;
import de.renew.unify.Impossible;

public interface EarlyExecutable extends Executable {
  // Get the phase during which this executable should execute.
  // Must be less than or equal to 0.
  long phase();

  // Lock the resources required to execute this executable.
  void lock() throws Impossible;

  // Execute this executable, then unlock the resources. 
  // Only called if the lock method succeeded.
  // Unlike the overwritten method from Executable,
  // this method must not throw an exception.
  void execute(Tracer tracer);

  // Unlock the resources without executing this executable.
  // Only called if the lock method succeeded.
  void unlock();
}
