package de.renew.simulator;

import de.renew.unify.*;

public class EqualsExpression implements Expression {
  Expression left;
  Expression right;

  public EqualsExpression(Expression left, Expression right) {
    this.left=left;
    this.right=right;
  }

  public boolean isInvertible() {
    return true;
  }

  public Object startEvaluation(VariableMapper mapper, StateRecorder recorder, 
    CalculationChecker checker) throws Impossible
  {
    Object leftResult=left.startEvaluation(mapper,recorder,checker);
    // The left result must be stored in a variable, because
    // it might be an unknown that changes its value
    // during the evluation of the right hand side.
    Variable result=new Variable(left.startEvaluation(mapper,recorder,checker),
      recorder);

    Object rightResult=right.startEvaluation(mapper,recorder,checker);

    Unify.unify(result,rightResult,recorder);

    return result.getValue();
  }

  public Object registerCalculation(VariableMapper mapper, 
    StateRecorder recorder, CalculationChecker checker) throws Impossible
  {
    // The left result must be stored in a variable, because
    // it might be an unknown that changes its value
    // during the evluation of the right hand side.
    Variable result=new Variable(
      left.registerCalculation(mapper,recorder,checker),recorder);
    Object rightObject=right.registerCalculation(mapper,recorder,checker);
    
    checker.addCalculated(result.getValue(),rightObject,recorder);

    return result.getValue();
  }
}

