package de.renew.simulator;

import collections.*;
import de.renew.unify.*;

class InputArcExecutable implements EarlyExecutable {
  PlaceInstance placeInstance;
  Object token;
  boolean trace;
  boolean releaseImmediately;

  InputArcExecutable(PlaceInstance placeInstance, Object token,
    boolean releaseImmediately, boolean trace)
  {
    this.placeInstance=placeInstance;
    this.token=token;
    this.trace=trace;
    // If releaseImmediately is true, the arc behaves like
    // a double arc that puts the token back as soon as possible.
    this.releaseImmediately=releaseImmediately;
  }

  public long phase() {
    return placeInstance.lockOrder;
  }

  public void lock() throws Impossible {
    placeInstance.lock.lock();
    try {
      placeInstance.removeToken(token);
    }
    catch (Exception e) {
      placeInstance.lock.unlock();
      // No reservation actually took place.
      throw new Impossible();
    }
  }

  public void execute(Tracer tracer) {
    if (releaseImmediately) {
      if (trace) {
	tracer.trace("    Checking "+token+" in "+placeInstance);
      }     
      placeInstance.insertToken(token);
    } else {
      if (trace) {
	tracer.trace("    Removing "+token+" from "+placeInstance);
      }     
    }
    placeInstance.lock.unlock();
  }

  public void unlock() {
    // We have to undo the previous removal. We cannot do this
    // without notifying the observers and listeners, because
    // it was not done silently.
    placeInstance.insertToken(token);
    placeInstance.lock.unlock();
  }
}
