package de.renew.simulator;

import collections.*;
import de.renew.unify.*;

class InputPlaceBinder implements Binder {
  Variable variable;
  TokenLister lister;
  TokenCollector collector;

  InputPlaceBinder(Variable variable,
    TokenLister lister, TokenCollector collector) 
  {
    this.variable=variable;
    this.lister=lister;
    this.collector=collector;
  }

  public int bindingBadness(Searcher searcher) {
    // The possibly incomplete value in the variable
    // will not be stored anywhere, so that it is
    // not required to make a copy of it.
    if (Unify.isBound(variable)) {
      if (collector.containsToken(variable.getValue())) {
	return 1;
      } else {
	return 0;
      }
    } else if (lister==null) {
      // We must not try to bind the variable.
      return BindingBadness.max;
    } else {
      lister.lock();
      CollectionEnumeration enum=lister.getDistinctTokens(variable.getValue());
      int result=BindingBadness.clip(enum.numberOfRemainingElements());  
      lister.unlock();
      return result;
    }
  }

  private void searchBound(Searcher searcher, Object rawValue) {
    // Here we must make copy of the value, because
    // the current value might be rolled back.
    // However, the new value must be created
    // without the possibility for rollback.
    // The value is completely bound anyway, so there
    // won't be a problem.
    Object value=Unify.copyBoundValue(rawValue);

    if (collector.collectToken(value)) {
      searcher.removeBinder(this);
      searcher.search();
      searcher.addBinder(this);
      collector.freeToken(value);
    }
  }

  public void bind(Searcher searcher) {
    // Make sure that the place instance notifies
    // the searchable if its marking changes,
    // because in that case the possible binding would have
    // to be rechecked. This is only required for input
    // arcs, because output arcs cannot disable or enable
    // a transition.
    collector.registerAt(searcher);

    if (Unify.isBound(variable)) {
      searchBound(searcher,variable.getValue());
    } else {
      lister.lock();
      CollectionEnumeration values=new RandomEnumeration(
	lister.getDistinctTokens(variable.getValue()));
      lister.unlock();

      while (values.hasMoreElements() && !searcher.isCompleted()) {
	Object value=values.nextElement();

	int checkpoint=searcher.recorder.checkpoint();
	try {
	  Unify.unify(variable,value,searcher.recorder);
	  // Ok, unification succeeded.
	  searchBound(searcher,variable.getValue());
	}
	catch (Impossible e) {
	  // Unification failed.
	}
	searcher.recorder.restore(checkpoint);
      }
    }
  }
}
