package de.renew.simulator;

import de.renew.unify.Impossible;
import collections.*;
import java.util.Hashtable;

public class Net {
  private static Hashtable netsByName=new Hashtable();
  public static Net forName(String name) {
    Net net=(Net)((Hashtable)netsByName).get(name);
    if (net==null) {
      System.err.println("Net "+name+" not loaded.");
    }
    return net;
  }

  UpdatableSet places;
  UpdatableSet transitions;

  String name;

  private int netCount;

  public Net(String name) {
    this.name=name;
    places=new HashedSet();
    transitions=new HashedSet();
    resetNetNumber();

    netsByName.put(name,this);
  }

  public void resetNetNumber() {
    netCount=0;
  }

  int makeNetNumber() {
    return ++netCount;
  }

  public String toString() {
    return name;
  }

  NetInstance makeInstance() throws Impossible {
    // Instead of creating a generic NetInstance, we might also
    // want to create a specialized NetInstance that supports
    // own methods and implements interfaces.
    return new NetInstanceImpl(this);
  }

  // Unlike the previous method, this publicly accessible
  // method immediately confirms the creation of the
  // generated net instance, so that external users do not need
  // to know about
  public NetInstance buildInstance() {
    try {
      NetInstance instance=makeInstance();
      instance.createConfirmation(SilentTracer.INSTANCE);
      return instance;
    } catch (Impossible e) {
      throw new RuntimeException("Could not make net instance.");
    }
  }

  void add(Place place) {
    places.include(place);
  }
  void add(Transition transition) {
    transitions.include(transition);
  }
  void remove(Place place) {
    places.exclude(place);
  }
  void remove(Transition transition) {
    transitions.exclude(transition);
  }
}

