package de.renew.simulator;

import java.util.Enumeration;
import collections.*;
import de.renew.unify.*;

public class NetChannelBinder implements Binder {
  // Try bindings with all the transitions possible
  // for the given net. Usually there will be very few
  // possible transitions.
  Variable netVariable;
  String name;
  Variable downlink;
  // If true, indicates that this channel need not be used if
  // the called net has no appropriate channels at all.
  boolean isOptional;

  public NetChannelBinder(Variable netVariable, String name, Variable downlink,
    boolean isOptional)
  {
    this.downlink=downlink;
    this.name=name;
    this.netVariable=netVariable;
    this.isOptional=isOptional;
  }

  private CollectionEnumeration getTransitions(NetInstance netInstance,
    String name)
  {
    UpdatableSeq result=new LinkedList();
    
    CollectionEnumeration enum=netInstance.getNet().transitions.elements();
    while (enum.hasMoreElements()) {
      Transition transition=(Transition)enum.nextElement();
      if (transition.uplink!=null) {
	if (transition.uplink.name.equals(name)) {
	  result.insertFirst(netInstance.getInstance(transition));
	}
      }
    }
    return result.elements();
  }

  public int bindingBadness(Searcher searcher) {
    Object value=netVariable.getValue();
    if (value instanceof NetInstance) {
      NetInstance net=(NetInstance)value;
      CollectionEnumeration transitions=
        getTransitions(net,name);
      return BindingBadness.clip(transitions.numberOfRemainingElements());
    } else {
      return BindingBadness.max;
    }
  }

  public void bind(Searcher searcher) {
    NetInstance net=
      (NetInstance)netVariable.getValue();
    CollectionEnumeration transitions=new RandomEnumeration(
      getTransitions(net,name));

    if (isOptional && !transitions.hasMoreElements()) {
      // This channel is optional and no channel is offered at
      // the invoked net. This is too easy.
      int checkpoint=searcher.recorder.checkpoint();
      searcher.removeBinder(this);
      searcher.search();
      searcher.addBinder(this);
      searcher.recorder.restore(checkpoint);
    } else {
      // Either the channel is not optional, or a suitable
      // channel is provided.
      while (transitions.hasMoreElements() && !searcher.isCompleted()) {
	TransitionInstance transitionInstance=(TransitionInstance)
	  transitions.nextElement();
	
	int checkpoint=searcher.recorder.checkpoint();
	
	try {
	  TransitionOccurrence occurrence=
	    new TransitionOccurrence(transitionInstance,downlink,searcher);

	  // Now try to find a binding.
	  searcher.addOccurrence(occurrence);
	  searcher.removeBinder(this);
	  searcher.addBinders(occurrence.getBinders());
	  searcher.search();
	  searcher.removeBinders(occurrence.getBinders());
	  searcher.addBinder(this);
	  searcher.removeOccurrence(occurrence);
	}
	catch (Impossible e) {
	  // This is an expected exception. Simply abandon this
	  // search branch and try to find another binding.
	}
	
	searcher.recorder.restore(checkpoint);
      }
    }
  }
}

