package de.renew.simulator;

import de.renew.unify.*;
import collections.*;

public class Place {
  PlaceType type;
  Net net;
  String name;

  /**
   * true, if my instances should output a trace message when they
   * occur. This is the default.
   **/
  boolean trace;

  /**
   * My set of initial marking inscriptions.
   **/
  UpdatableSet inscriptions;

  public Place(Net net, PlaceType type, String name) {
    this.net=net;
    this.type=type;
    this.name=name;

    trace=true;
    inscriptions=new HashedSet();

    net.add(this);
  }
  public Place(Net net, String name) {
    // Generate a place that cannot hint at allowed values.
    this(net,new PlaceType(),name);
  }

  public String toString() {
    return name;
  }

  PlaceInstance makeInstance(NetInstance netInstance) 
    throws Impossible
  {
    PlaceInstance instance=new PlaceInstance(netInstance,this);
    
    return instance;
  }

  /**
   * Switch my trace flag on or off.
   *
   * @param trace true, if tracing is desired
   **/
  public void setTrace(boolean trace) {
    this.trace=trace;
  }

  /**
   * Am I being traced?
   *
   * @return true, if my trace flag is set
   **/
  public boolean getTrace() {
    return trace;
  }

  /**
   * Give me another initial marking.
   **/
  public void add(InitialMarking initialMarking) {
    inscriptions.include(initialMarking);
  }
  
  /**
   * Remove one my current initial markings.
   **/
  public void remove(InitialMarking initialMarking) {
    inscriptions.exclude(initialMarking);
  }
}

