package de.renew.simulator;

import collections.*;

/**
 * I am the token listener interface. Upon request,
 * my childern will provide a set of object that
 * are contained in a place. They might make special
 * restrictions on the tokens, e.g., only a special type,
 * only testable tokens, only free tokens, etc.
 **/
public interface TokenLister {
  /**
   * I provide a set of tokens that might match the possibly
   * incomplete pattern. I will often return tokens that
   * do not match the pattern, but I promise to do it fast.
   *
   * @return an enumeration with the possible different tokens.
   **/
  public CollectionEnumeration getDistinctTokens(Object pattern);
  /**
   * I can lock the token source, so that the enumeration
   * remains intact during its processing. It is suggested
   * to copy the enumeration into a new data structure if
   * it is used for a long time.
   **/
  public void lock();
  /**
   * I will only unlock the token source at your request.
   **/
  public void unlock();
}
