package de.renew.simulator;

import collections.*;
import java.util.Hashtable;

// The token reserver is responsible for simulating
// the state of several place instances that would occur if
// a number of token removals and tests was done.

class TokenReserver {
  Hashtable reservations;

  TokenReserver() {
    reservations=new Hashtable();
  }

  public void reset() {
    // In theory, a reservations.clear() should suffice
    // at this point. For some entirely obscure point, however,
    // the Java garbage collector does not like it.
    // This is annoying.
    reservations=new Hashtable();
  }

  private TokenReservation getReservation(PlaceInstance place) {
    if (reservations.containsKey(place)) {
      return (TokenReservation)reservations.get(place);
    } else {
      TokenReservation reservation=new TokenReservation(place);
      reservations.put(place,reservation);
      return reservation;
    }
  }

  private void disposeReservation(PlaceInstance place) {
    TokenReservation reservation=(TokenReservation)reservations.get(place);
    if (reservation.isRemovable()) {
      reservations.remove(place);
    }
  }
  
  public synchronized boolean containsRemovableToken(PlaceInstance place,
    Object token)
  {
    TokenReservation reservation=getReservation(place);
    boolean result=reservation.containsRemovableToken(token);
    disposeReservation(place);
    return result;
  }

  public synchronized boolean containsTestableToken(PlaceInstance place,
    Object token)
  {
    TokenReservation reservation=getReservation(place);
    boolean result=reservation.containsTestableToken(token);
    disposeReservation(place);
    return result;
  }

  public synchronized boolean removeToken(PlaceInstance place,
    Object token)
  {
    TokenReservation reservation=getReservation(place);
    boolean result=reservation.removeToken(token);
    disposeReservation(place);
    return result;
  }

  public synchronized boolean testToken(PlaceInstance place,
    Object token)
  {
    TokenReservation reservation=getReservation(place);
    boolean result=reservation.testToken(token);
    disposeReservation(place);
    return result;
  }

  public synchronized void unremoveToken(PlaceInstance place,
    Object token)
  {
    TokenReservation reservation=getReservation(place);
    reservation.unremoveToken(token);
    disposeReservation(place);
  }

  public synchronized void untestToken(PlaceInstance place,
    Object token)
  {
    TokenReservation reservation=getReservation(place);
    reservation.untestToken(token);
    disposeReservation(place);
  }

  public synchronized CollectionEnumeration getRemovableTokens(
    PlaceInstance place, Object pattern)
  {
    TokenReservation reservation=getReservation(place);
    CollectionEnumeration result=reservation.getRemovableTokens(pattern);
    disposeReservation(place);
    return result;
  }

  public synchronized CollectionEnumeration getTestableTokens(
    PlaceInstance place, Object pattern)
  {
    TokenReservation reservation=getReservation(place);
    CollectionEnumeration result=reservation.getTestableTokens(pattern);
    disposeReservation(place);
    return result;
  }
}
