package de.renew.simulator;

import collections.*;

/**
 * I am a special executable. I will output a trace message
 * during execution. I am useful for debugging purposes
 * and for console traces of firing sequences.
 **/
class TraceExecutable implements EarlyExecutable {
  /**
   * The message that I must output.
   **/
  String message;

  /**
   * I (a trace executable) am created. I store the argument
   * message for further procession during the execute phase.
   *
   * @param message
   *   the message that I must output
   **/
  TraceExecutable(String message) {
    this.message=message;
  }

  /**
   * I will always act during the phase 0. Maybe at some
   * time this could be configured.
   *
   * @returns 0 (the phase that I must await)
   **/
  public long phase() {
    return 0;
  }

  /**
   * Lock required resources. Nothing to do.
   **/
  public void lock() {}

  /**
   * I output my message using the tracer provided as an
   * argument.
   *
   * @param tracer the tracer that want to receive my message.
   **/
  public void execute(Tracer tracer) {
    tracer.trace(message);
  }

  /**
   * Undo previous actions. Nothing to do.
   **/
  public void undo() {}

  /**
   * Unlock required resources. Nothing to do.
   **/
  public void unlock() {}
}
