package de.renew.simulator;

import collections.*;

public class TriggerCollection {
  // Keep track of triggering objects.
  private UpdatableSet triggers;
  private Searchable searchable;

  public TriggerCollection(Searchable searchable) {
    triggers=new HashedSet();
    this.searchable=searchable;
  }

  // The following two method must only be accessed through
  // the include and exclude methods of searchable collections,
  // otherwise deadlocks might occur. Those method will make sure 
  // to lock this object.
  public void include(SearchableCollection trigger) {
    triggers.include(trigger);
  }

  public void exclude(SearchableCollection trigger) {
    triggers.exclude(trigger);
  }

  public void clear() {
    LinkedList list=new LinkedList();
    synchronized(this) {
      // Get the triggers.
      list.prependElements(triggers.elements());
    }

    // Send the notifications. No need to synchronize here.
    // Better allow concurrent accesses to reduce deadlock potential.
    // The searchable collections are required to do the locking.
    CollectionEnumeration enum=list.elements();
    while (enum.hasMoreElements()) {
      SearchableCollection searchables=
	(SearchableCollection)enum.nextElement();
      searchables.exclude(searchable);
    }
  }
}
