package de.renew.simulator;

import de.renew.unify.*;

/**
 * I represent variables in expressions. Upon evaluation
 * I make sure to map the local variable I was given
 * at creation time to the value of this variable.
 *
 * @author Olaf Kummer
 **/
public class VariableExpression implements Expression {
  /**
   * Here I store the name of the local variable
   * that I must evaluate.
   **/
  LocalVariable localVariable;

  /**
   * I (a variable expression) am created for a given local variable.
   *
   * @param localVariable 
   *   The local variable that should be mapped during evaluation.
   **/
  public VariableExpression(LocalVariable localVariable) {
    this.localVariable=localVariable;
  }

  /**
   * Am I invertible? Of course I am.
   *
   * @return true
   **/
  public boolean isInvertible() {
    return true;
  }

  /**
   * I will return either the value of my associated variable
   * or an appropriate unknown object.
   *
   * @return the current variable value
   **/
  public Object startEvaluation(VariableMapper mapper, StateRecorder recorder, 
    CalculationChecker checker) throws Impossible
  {
    return mapper.map(localVariable).getValue();
  }

  /**
   * I will return either the value of my associated variable
   * or an appropriate unknown object. Unlike my colleagues,
   * I need not make a distinction between evaluation and
   * registration.
   *
   * @return the current variable value
   **/
  public Object registerCalculation(VariableMapper mapper,
    StateRecorder recorder, CalculationChecker checker) throws Impossible
  {
    return mapper.map(localVariable).getValue();
  }
}

