package de.renew.simulator;

import java.util.Enumeration;
import java.util.Hashtable;
import de.renew.unify.*;

/**
 * I map local variables to their respective actual
 * values in the current context.
 *
 * @author Olaf Kummer
 **/
public class VariableMapper {
  /**
   * This hashtable does the mapping for those variables
   * that I met so far.
   **/
  private Hashtable hashtable;

  /**
   * I (a variable mapper) am created. I map no variables so far.
   **/
  VariableMapper() {
    hashtable=new Hashtable();
  }

  /**
   * Create a new instance that already
   * maps all the local variables that are
   * known to me. Assume that I would return a certain
   * variable. If that variable is send through the copier
   * you get another variable that the new instance will return
   * after the same query.
   *
   * Put differently, you get a functional composition
   * of the copier and myself.
   *
   * @param copier
   *   the copier that maps my variables to their copies
   * @return the new variable mapper
   **/
  VariableMapper makeCopy(Copier copier) {
    VariableMapper mapper=new VariableMapper();
    Enumeration enum=hashtable.keys();
    while (enum.hasMoreElements()) {
      Object key=enum.nextElement();
      mapper.hashtable.put(key,
        copier.copy(hashtable.get(key)));
    }
    return mapper;
  }

  /**
   * I will map the local variable to a variable.
   * On the first call with some argument value, I return a fresh
   * variable. On successive calls with the same argument value,
   * I will always be consistent and return the original variable
   * again.
   **/
  Variable map(LocalVariable localVariable) {
    Variable variable=(Variable)hashtable.get(localVariable);
    if (variable==null) {
      variable=new Variable();
      hashtable.put(localVariable,variable);
    }
    return variable;
  }

  /**
   * I can return all local variables that I have mapped so far.
   **/
  Enumeration getLocalVariables() {
    return hashtable.keys();
  }
}


