package de.renew.unify;

import de.renew.util.HashedRelation;
import collections.CollectionEnumeration;

public class ArityBranch {
  HashedRelation relation=new HashedRelation();
  private ComponentBranch[] branches=new ComponentBranch[0];

  public ArityBranch() {}

  private void setCapacity(int cap) {
    ComponentBranch[] newBranches=new ComponentBranch[cap];
    System.arraycopy(branches,0,newBranches,0,Math.min(cap,branches.length));
    branches=newBranches;  
  }

  private void ensureCapacity(int cap) {
    if (branches.length<cap) {
      setCapacity(cap);
    }
  }

  private void trim() {
    int cap=branches.length;
    while (cap>0 && branches[cap-1]==null) {
      cap--;
    }
    if (cap<branches.length) {
      setCapacity(cap);
    }
  }

  void traverse(TupleIndexVisitor visitor, Object remainder) {
    if (visitor.visitIndex(this,remainder)) {
      Tuple tuple=null;
      int arity=0;
      
      if (remainder instanceof Tuple) {
	tuple=(Tuple)remainder;
	arity=tuple.getArity();
      }
      
      // Make sure to have something traversable.
      ensureCapacity(arity+1);
      if (branches[arity]==null) {
	branches[arity]=new ComponentBranch(arity);
      }

      // Traverse.
      branches[arity].traverse(visitor,tuple);

      // Make sure to clean up, if nothing stored.
      if (branches[arity].size==0) {
	branches[arity]=null;
	trim();
      }
    }
  }
}
