package de.renew.unify;

import collections.*;

class BacklinkSet {
  /**
   * Up to six backlinked objects can be handled without
   * a special data structure.
   **/
  private int cnt=0;
  private Reference[] backlinks=new Reference[6];
  private UpdatableSet externalSet=null;

  private boolean includes(Reference reference) {
    if (externalSet!=null) {
      return externalSet.includes(new IdentityWrapper(reference));
    } else {
      switch (cnt) {
      case 6: return reference==backlinks[0] || reference==backlinks[1]
		|| reference==backlinks[2] || reference==backlinks[3]
		|| reference==backlinks[4] || reference==backlinks[5];
      case 5: return reference==backlinks[0] || reference==backlinks[1]
		|| reference==backlinks[2] || reference==backlinks[3]
		|| reference==backlinks[4];
      case 4: return reference==backlinks[0] || reference==backlinks[1]
		|| reference==backlinks[2] || reference==backlinks[3];
      case 3: return reference==backlinks[0] || reference==backlinks[1]
		|| reference==backlinks[2];
      case 2: return reference==backlinks[0] || reference==backlinks[1];
      case 1: return reference==backlinks[0];
      }
      return false;
    }
  }

  /**
   * Include an element. This element must not be
   * included in the set so far.
   **/
  private void includeNonElement(Reference reference) {
    if (externalSet==null && cnt==6) {
      externalSet=new HashedSet();
      for (int i=0;i<6;i++) {
	externalSet.include(new IdentityWrapper(backlinks[i]));
      }
    }
    if (externalSet!=null) {
      externalSet.include(new IdentityWrapper(reference));
    } else {
      backlinks[cnt++]=reference;
    }
  }

  /**
   * Exclude an element. This element must be
   * included in the set.
   **/
  private void exclude(Reference reference) {
    if (externalSet!=null) {
      // I do not switch back to array once I have created
      // the hashed set. It is probable that the number
      // of links will grow again in this case.
      externalSet.exclude(new IdentityWrapper(reference));
    } else {
      cnt--;
      if (backlinks[cnt]!=reference) {
	if (backlinks[0]==reference) {
	  backlinks[0]=backlinks[cnt];
	} else if (backlinks[1]==reference) {
	  backlinks[1]=backlinks[cnt];
	} else if (backlinks[2]==reference) {
	  backlinks[2]=backlinks[cnt];
	} else {
	  throw new RuntimeException("Backlink not found. Strange. ");
	}
      }

      // Enable garbage collection.
      backlinks[cnt]=null;
    }
  }

  void addBacklink(final Reference reference, StateRecorder recorder) {
    if (!includes(reference)) {
      includeNonElement(reference);
      if (recorder!=null) {
	recorder.record(new StateRestorer() {
	  public void restore() {
	    exclude(reference);
	  }
	});
      }
    }
  }

  void updateBacklinked(Referable oldValue, Object newValue,
    UpdatableSet listeners, StateRecorder recorder) throws Impossible
  {
    // I will tell all my backlinked objects that the given
    // value is now relevant instead of myself. (Maybe I *am* the
    // new object, but who knows.)
    //
    // I assume that no new backlinks will be added to myself
    // during this procedure.
    if (externalSet!=null) {
      CollectionEnumeration enum=externalSet.elements();
      while (enum.hasMoreElements()) {
	IdentityWrapper wrapper=(IdentityWrapper)enum.nextElement();
	Reference reference=(Reference)wrapper.getObject();
	reference.update(oldValue,newValue,listeners,recorder);
      }
    } else {
      for (int i=0;i<cnt;i++) {
	backlinks[i].update(oldValue,newValue,listeners,recorder);
      } 
    }
  }  
}
