package de.renew.unify;

import collections.*;

final class Calculator implements Referable, Referer {
  final private BacklinkSet backlinkSet=new BacklinkSet();
  final private Reference reference;
  final private RecorderChecker recorderChecker;
 
  Calculator(Object object, StateRecorder recorder) {
    recorderChecker=new RecorderChecker(recorder);
    reference=new Reference(object,this,recorder);
  }

  public boolean isComplete() {
    return reference.complete;
  }

  public boolean isBound() {
    return false;
  }  

  public void addBacklink(Reference reference, StateRecorder recorder) {
    recorderChecker.checkRecorder(recorder);
    backlinkSet.addBacklink(reference,recorder);
  }

  public void possiblyCompleted(UpdatableSet listeners, StateRecorder recorder)
    throws Impossible
  {
    // I own only one reference, therefore I am guaranteed to be completed.
    backlinkSet.updateBacklinked((Referable)this,this,listeners,recorder);
  }

  public void occursCheck(Unknown that, UpdatableSet visited)
    throws Impossible
  {
    // Am I complete? If yes, no unknown can possibly
    // be contained within me.
    if (reference.complete) {
      return;
    }

    // Did I check myself earlier?
    IdentityWrapper thisWrapper=new IdentityWrapper(this);
    if (visited.includes(thisWrapper)) {
      return;
    }
    // I do not want to check me again.
    visited.include(thisWrapper);

    reference.occursCheck(that,visited);
  }

  // Hide the normal equal and hashCode methods.
  // Because calculators are not values, it would
  // be desastrous to insert them into a hashtable.
  public boolean equal(Object that) {
    throw new RuntimeException(
      "Somebody compared a calculator.");
  }
  public int hashCode() {
    throw new RuntimeException(
      "Somebody took the hash code of a calculator.");
  }
}
