package de.renew.unify;

import collections.*;

class Reference {
  Object value;
  boolean bound;
  boolean complete;

  Referer referer;

  Reference(Object value, Referer referer, StateRecorder recorder) {
    // Get rid of variables.
    if (value instanceof Variable) {
      value=((Variable)value).getValue();
    }
    // Initialize.
    this.value=value;
    complete=Unify.isComplete(value);
    bound=Unify.isBound(value);

    this.referer=referer;
    if (value instanceof Referable) {
      ((Referable)value).addBacklink(this,recorder);
    }
  }

  public void occursCheck(Unknown that, UpdatableSet visited)
    throws Impossible
  {
    // Now let's test the referenced object.
    if (value instanceof Referable) {
      ((Referable)value).occursCheck(that,visited);
    }
  }

  void update(final Referable oldValue, final Object newValue,
    UpdatableSet listeners, StateRecorder recorder) throws Impossible
  {
    // A strange phenomenon: The completion of a subobject might
    // be indicated multiple times. Scenario: A=[B,C], B=[C]
    // and C becomes complete. Now C notifies B, which becomes
    // complete and notifies A in turn, which is complete now, too.
    // Now the notification from C to A (C occurs in A)
    // remains to be done, but A is already complete. Therefore
    // I must be prepared to be notified although I am already
    // complete. I simply discard any late notifications.

    if (!complete) {
      if (value==oldValue && oldValue!=newValue) {
	if (recorder!=null) {
	  recorder.record(new StateRestorer() {
	    public void restore() {
	      value=oldValue;
	    }
	  });
	}
	value=newValue;
	if (value instanceof Referable) {
	  ((Referable)value).addBacklink(this,recorder);
	}
      }
      
      if (Unify.isComplete(value)) {
 	if (recorder!=null) {
	  recorder.record(new StateRestorer() {
	    public void restore() {
	      complete=false;
	    }
	  });
	}
	complete=true;
     
	if (Unify.isBound(value)) {
	  if (recorder!=null) {
	    recorder.record(new StateRestorer() {
	      public void restore() {
		bound=false;
	      }
	    });
	  }
	  bound=true;
	}

	// Notify the referer about the change of completeness.
	referer.possiblyCompleted(listeners,recorder);
      }	
    }
  }
}
