package de.renew.unify;

import collections.*;
import de.renew.util.Null;

public class TupleIndex {
  ArityBranch tree=new ArityBranch();

  public TupleIndex() {}

  // Convert nulls?
  //
  // public CollectionEnumeration elements() {
  //    return tree.relation.keys();
  // }  

  public synchronized void insert(final Object element) {
    final Object elem=Null.convert(element);
    tree.traverse(new TupleIndexVisitor() {
      public void visitBranch(ComponentBranch branch) {
	branch.size++;
	branch.all.include(elem);
      }
      public boolean visitIndex(ArityBranch branch, Object remainder) {
	branch.relation.put(Null.convert(remainder),elem);
	return true;
      }
    },element);
  }

  public synchronized void remove(final Object element) {
    final Object elem=Null.convert(element);
    tree.traverse(new TupleIndexVisitor() {
      public void visitBranch(ComponentBranch branch) {
	branch.size--;
	branch.all.removeOneOf(elem);
      }
      public boolean visitIndex(ArityBranch branch, Object remainder) {
	branch.relation.remove(Null.convert(remainder),elem);
	return true;
      }
    },element);
  }

  public CollectionEnumeration getAllElements() {
    return Null.convertEnumeration(tree.relation.keys());
  }

  public CollectionEnumeration getPossibleMatches(Object pattern) {
    EstimateVisitor visitor=new EstimateVisitor();
    tree.traverse(visitor,pattern);
    visitor.possibleEnumeration(tree.relation.keys());

    return Null.convertEnumeration(visitor.bestEnumeration);
  }

  private static void test(TupleIndex index, Object pattern) {
    CollectionEnumeration result=index.getPossibleMatches(pattern);
    System.out.println("Searching");
    while (result.hasMoreElements()) {
      System.out.println("       found: "+result.nextElement());
    }
  }

  public static void main(String[] args) {
    TupleIndex index=new TupleIndex();
    Tuple t1=new Tuple(new Object[]{"a",new Object(),Boolean.TRUE},null);
    Tuple t2=new Tuple(new Object[]{
      new Tuple(new Object[]{"kuno"},null)},null);
    index.insert(t1);
    index.insert(t2);
    index.insert(new Tuple(0));
    index.insert(new Integer(1));
    index.insert(new Integer(2));
    test(index,t1);
    test(index,new Unknown());
    test(index,Boolean.FALSE);
    test(index,new Integer(2));
    test(index,new Tuple(3));

    index=new TupleIndex();
    index.insert(new Tuple(new Object[]{"a","a","a"},null));
    index.insert(new Tuple(new Object[]{"a","a","b"},null));
    index.insert(new Tuple(new Object[]{"a","a","c"},null));
    index.insert(new Tuple(new Object[]{"c","a","c"},null));
    test(index,new Tuple(new Object[]{"a",new Unknown(),"c"},null));
    test(index,new Tuple(new Object[]{"a","a","c"},null));
    test(index,new Tuple(new Object[]{"c","a","a"},null));
  }
}
