package de.renew.unify;

import collections.*;

public final class Variable implements Referer {
  final private UpdatableSet myListeners=new HashedSet();
  final private Reference reference;
  final private RecorderChecker recorderChecker;

  public Variable() {
    recorderChecker=new RecorderChecker(null);
    reference=new Reference(new Unknown(),this,null);
  }
  
  public Variable(Object initValue, StateRecorder recorder) {
    recorderChecker=new RecorderChecker(recorder);
    reference=new Reference(initValue,this,recorder);
  }

  public boolean isComplete() {
    return reference.complete;
  }

  public boolean isBound() {
    return reference.bound;
  }

  public void possiblyCompleted(UpdatableSet listeners, StateRecorder recorder)
    throws Impossible
  {
    // I own only one reference, therefore I am guaranteed to be completed.
    // Am I bound, too?
    if (isBound()) {
      listeners.includeElements(myListeners.elements());
    }
  }

  public Object getValue() {
    return reference.value;
  }

  public void addListener(final Notifiable listener, StateRecorder recorder)
    throws Impossible
  {
    recorderChecker.checkRecorder(recorder);

    if (!myListeners.includes(listener)) {
      myListeners.include(listener);
      if (recorder!=null) {
	recorder.record(new StateRestorer() {
	  public void restore() {
	    myListeners.exclude(listener);
	  }
	});
      }
    
      // If I am already bound, i will not have another opportunity to
      // send a notification. I'll do it now, before it's too late.
      if (isBound()) {
	listener.boundNotify(recorder);
      }
    }
  }
}
