package de.renew.util;

// In essence this class duplicates the functionality of the
// synchronized statement. But locks can be used without
// a textual matching in the source code, which allows
// more flexible (and hence more dangerous) synchronisation
// schemes.

public class Lock {
  private Thread lockingThread;
  private int lockCount;
  public Lock() {
    lockingThread=null;
    lockCount=0;
  }
  public synchronized void lock() {
    Thread currentThread=Thread.currentThread();
    while (lockingThread!=null && lockingThread!=currentThread) {
      try {
	wait();
      } catch (InterruptedException e) {}
    }
    
    lockingThread=currentThread;
    lockCount++;
  }
  public synchronized void unlock() {
    Thread currentThread=Thread.currentThread();
    if (currentThread!=lockingThread) {
      throw new RuntimeException("Lock/unlock mismatch.");
    }
    lockCount--;
    if (lockCount==0) {
      lockingThread=null;
    }
    notify();
  }
}
