package fs;

import collections.*;
import java.util.*;
import de.renew.function.*;

public class Bound implements Concept {

  public final static int LOWER=1;
  public final static int UPPER=-1;

  private Object bound;
  private int type;

  public Bound(int type, Object bound) {
     if ((type == UPPER || type == LOWER)  && (bound instanceof Value || bound instanceof String)) {
        this.bound=bound;
        this.type=type;
     } else {
        System.err.println("Type: "+type+" Class of bound: "+bound.getClass());
        throw new RuntimeException("Wrong arguments in constructor of Bound.");
     }
  }

  public boolean subsumesObject(Object javaObject) {
     try {
        return compare(bound,javaObject)!=type;
     } catch (TypeException tee) {
        return false;
     }
  }

  /** Return the name of this Concept. */
  public String getName() {
     String relation = type == LOWER ? ">=" : "<=";
     if (bound instanceof String)
        return relation+'"' + bound.toString() + '"';
     else 
        return relation+((Value)bound).value.toString();
  }

  public String toString() { return getName(); }

  /** Return whether this Concept is extensional. */
  public boolean isExtensional() { return false; }

  /** Return whether the feature <feature> is appropriate in this Concept. 
   */
  public boolean isApprop(Integer featureCode) { return false; }

  /** Return the required Type of the Value found under the given feature. */
  public Type appropType(Integer featureCode) throws NoSuchElementException {
     throw new NoSuchElementException();
  }

  /** Return an Enumeration of all appropriate features. */
  public CollectionEnumeration appropFeatureCodes() {
      return new HashedSet().elements();
  }

  /** Return whether this Concept is-a <that> Concept.
    * In other words, return wheter this Concept is more special than <that>
    * Concept.
    */
  public boolean isa(Concept that) {
     try {
        if (that instanceof Bound) {
           Bound thatBound=(Bound)that;
           return type == thatBound.type && compare(bound,thatBound.bound) != type;
        } else if (that instanceof JavaConcept) {
           Class clazz=((JavaConcept)that).getJavaClass();
           if (clazz.isInstance(bound)) {
              return true;
           } else if (bound instanceof Value) {
              return clazz.equals(de.renew.util.Types.typify(((Value)bound).value.getClass()));
           }
        }
     } catch (TypeException tee) { }
     return false;
  }

  /** Return whether this Concept is-not-a <that> Concept.
    * In other words, return wheter this Concept is incompatible with <that>
    * Concept.
    */
  public boolean isNotA(Concept that) {
     try {
        if (that instanceof Bound) {
           Bound thatBound=(Bound)that;
           return type != thatBound.type && compare(bound,thatBound.bound) == type;
        }
     } catch (TypeException tee) {
         return true;
     }
     return !(isa(that) || that.isa(this));
  }

/*
  private int compare(Object a, Object b) throws TypeException {
     System.err.println("Comparing "+a+" and "+b);
     int result=compare2(a,b);
     System.err.println("Result: "+result);
     return result;
  }
*/
      
  int compare(Object a, Object b) throws TypeException {
     if (a instanceof String && b instanceof String) {
        return ((String)a).compareTo((String)b);
     }
     if (a instanceof Value && b instanceof Value) {
        Value av=(Value)a, bv=(Value)b;
        Object ao=av.value, bo=bv.value;
        if (!ao.getClass().equals(bo.getClass())) {
           System.err.println("Different Value Classes: "+ao.getClass()+" and "+bo.getClass());
           throw new TypeException();
        }
        if (ao.equals(bo))
           return 0;
        if (ao instanceof Boolean) return bv.booleanValue() ? -1 : 1;
        else if (ao instanceof Character) return av.charValue()<bv.charValue() ? -1 : 1;
        else if (ao instanceof Byte) return av.byteValue()<bv.byteValue() ? -1 : 1;
        else if (ao instanceof Short) return av.shortValue()<bv.shortValue() ? -1 : 1;
        else if (ao instanceof Integer) return av.intValue()<bv.intValue() ? -1 : 1;
        else if (ao instanceof Long) return av.longValue()<bv.longValue() ? -1 : 1;
        else if (ao instanceof Float) return av.floatValue()<bv.floatValue() ? -1 : 1;
        else if (ao instanceof Double) return av.doubleValue()<bv.doubleValue() ? -1 : 1;
        else {
            System.err.println("Wrong bound classes: "+ao.getClass()+" and/or "+bo.getClass());
            throw new TypeException();
        }

     }
     throw new TypeException();
  }

  public CollectionEnumeration extensionalSuperconcepts() {
      return new HashedSet().elements();
  }

  public void remove() {}


  /** Return an XML representation of this Concept.
   */
   public String toXML() {
      return "<BOUND/>";
   }

  /** Return an XML representation of a reference to this Concept.
   */
  public String toXMLref() {
      return "<BOUNDREF/>";
  }

}
