package fs;

import java.io.*;
import java.awt.*;
import java.util.Vector;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.figures.TextFigure;
import CH.ifa.draw.util.ColorMap;
import CH.ifa.draw.standard.*;

public class ConceptFigure extends TextFigure {

  transient protected ConceptImpl concept = null;
  transient protected boolean addsFeatures = false;

  public ConceptFigure() {
    super();
    setAttribute("FrameColor", ColorMap.color("Black"));
    setAttribute("FillColor", ColorMap.color("White"));
  }

  public Vector handles() {
    Vector handles = super.handles();
    handles.addElement(new ConnectionHandle(this, RelativeLocator.north(),
					    new IsaConnection())
		       );
    return handles;
  }

  /* hack for setting ConceptFigures to editable: */
  public void read(CH.ifa.draw.util.StorableInput dr) throws IOException {
    super.read(dr);
    setReadOnly(false);
  }

   public void drawFrame(Graphics g) {
       super.drawFrame(g);
       if (addsFeatures) {
         /* separate the first line (name of concept) from the features: */
        Rectangle r = displayBox();
        FontMetrics metrics = g.getFontMetrics(getFont());
        int y = r.y+metrics.getHeight();
        g.drawLine(r.x, y, r.x+r.width-1, y);
      }
    }



  public ConceptImpl getConcept() { return concept; }

  public Rectangle displayBox() {
    Rectangle box = super.displayBox();
    return new Rectangle(box.x-5, box.y, box.width+10, box.height);
  }

  public void setText(String newText) {
    addsFeatures=false;
    try {
      // read lines:
      BufferedReader reader = new BufferedReader(new StringReader(newText));
      String conceptStr = reader.readLine().trim();
      if (conceptStr != null) {
	System.err.println("Concept: "+conceptStr);
        if (concept == null)
           concept  = new ConceptImpl(null);
        if (concept.setName(conceptStr)) {
	  concept.removeApprops();
	  String appropStr;
	  do {
	    appropStr = reader.readLine();
	    if (appropStr == null)
	      break;
	    appropStr = appropStr.trim();
	    if (appropStr.length()>0) {
              System.err.println("Approp: "+appropStr);
	      concept.addApprop(appropStr);
              addsFeatures=true;
            }
	  } while (true);
	} else {
	  System.err.println("Concept "+conceptStr+" already defined!");
	  newText = "*"+newText;
	}
      }
    } catch (IOException ex) { newText = "*" + newText; }
    super.setText(newText);
    //System.err.println("new Concept "+newText);
    //    setReadOnly(true);
  }

  public void release() {
    super.release();
    if (concept != null) {
      concept.remove();
      concept = null;
    }
  }

  public boolean canBeParent(ParentFigure figure) {
    if (figure instanceof PartitionFigure)
      return super.canBeParent(figure);
    // only PartitionFigures can be parents of ConceptFigures!
    return false;
  }

}
