package fs;

import CH.ifa.draw.figures.*;
import java.awt.*;
import collections.*;
import java.io.*;

public class FSFigure extends TextFigure {

  private Node fs;
  private TagMap tagmap;

  public FSFigure(FeatureStructure fs) {
    this.fs = fs.getRoot();
    tagmap = new TagMap(this.fs);
    setText(" ");
  }

  private void drawBrackets(Graphics g, int d, Rectangle r) {
    int yu = r.y + r.height-1, xr = r.x + r.width-1;
    g.drawPolyline(new int[] {r.x+d,r.x,r.x,r.x+d},
                   new int[] {r.y, r.y, yu, yu}, 4);
    g.drawPolyline(new int[] {xr-d, xr, xr, xr-d},
                   new int[] {r.y, r.y, yu, yu}, 4);
  }

  private void drawAngles(Graphics g, int d, Rectangle r) {
    int yu = r.y + r.height-1,
      ym = r.y + r.height/2,
      xr = r.x + r.width-1;
    g.drawPolyline(new int[] {r.x+d,r.x,r.x+d},
                   new int[] {r.y, ym, yu}, 3);
    g.drawPolyline(new int[] {xr-d, xr, xr-d },
                   new int[] {r.y, ym, yu}, 3);
  }

  private Dimension fExtent=null;

  public Rectangle displayBox() {
    Rectangle box = super.displayBox();
    if (fExtent == null) {
      FontMetrics metrics = Toolkit.getDefaultToolkit().getFontMetrics(getFont());
      tagmap = new TagMap(this.fs);
      fExtent = drawFS(null,metrics,box.x,box.y,fs);
    }
    return new Rectangle(box.x, box.y, fExtent.width, fExtent.height);
  }


  public void draw(Graphics g) {
    g.setColor((Color)getAttribute("TextColor"));
    g.setFont(getFont());
    FontMetrics metrics = g.getFontMetrics(getFont());
    Rectangle box = super.displayBox();
    tagmap.resetVisited();
    fExtent = drawFS(g,metrics,box.x,box.y,fs);
  }
  

  private Dimension drawList(Graphics g, FontMetrics metrics,
			     int x, int y, int maxw,
			     Node fs) {
    int lineh =  metrics.getHeight(), d = metrics.stringWidth(" "),
         h = 0;
    Type nodetype = fs.getType();
    if (Type.NELIST.subsumes(nodetype)) {
      Node head;
      if (fs.hasFeature(Feature.HEAD))
	head = fs.delta(Feature.HEAD);
      else
	head = new FSNode();
      Dimension headdim = drawFS(g,metrics,x,y,head);
      h = headdim.height + d;
      maxw = Math.max(maxw,headdim.width);

      Node tail;
      if (fs.hasFeature(Feature.TAIL))
	tail = fs.delta(Feature.TAIL);
      else
	tail = new FSNode();
      Dimension taildim = drawList(g,metrics,x,y+h,maxw,tail);
      h += taildim.height;
      maxw = Math.max(maxw,taildim.width);
    } else if (!Type.ELIST.subsumes(nodetype)) {
	Dimension restdim = drawFS(g,metrics,x,y + 2*d,fs);
	h = restdim.height + 2*d;
	maxw = Math.max(maxw,restdim.width);
	if (g != null)
	  g.drawLine(x,y+d,x+maxw,y+d);
    }
    return new Dimension(maxw,h);
  }

  private Dimension drawFS(Graphics g, FontMetrics metrics,
			   int x, int y, Node fs) {
    int ascent = metrics.getAscent(), lineh =  metrics.getHeight(),
      d = metrics.stringWidth(" "), h = lineh, maxw = d;
    Dimension extent;

    boolean isList = false;
    int tag = tagmap.getTag(fs);
    int tagwidth = 0;
    String tagstr=null;
    if (tag != 0) {
       tagstr = String.valueOf(tag);
       tagwidth = 2*d + metrics.stringWidth(tagstr);
	x += tagwidth;
    }
    int left = x+d;
    if (tagmap.isVisited(fs)) {
      extent = new Dimension(0,lineh);
    } else {
      tagmap.setVisited(fs);
      Type nodetype = fs.getType();
      if (Type.LIST.subsumes(nodetype)) {
	if (nodetype != Type.LIST) {
	  Dimension listdim = drawList(g,metrics,left+2*d,y,d,fs);
	  h = listdim.height;
	  if (h == 0)
	    h = lineh;
	  maxw = listdim.width + 4*d;
	  isList = true;
	}
      } else {
	if (nodetype != Type.TOP) {
	  String typestr = nodetype.toString();
	  if (g != null)
	    g.drawString(typestr, left, y+ascent);
	  maxw = metrics.stringWidth(typestr);
	} else
	  h = 0;
	CollectionEnumeration features = fs.featureCodes();
	if (features.hasMoreElements())
	  h += d;
	else
	  h = lineh;
	while (features.hasMoreElements()) {
	  Integer featureCode = (Integer)features.nextElement();
	  String feature = Feature.getFeature(featureCode) + ":";
	  int indent = d + metrics.stringWidth(feature);
	  Dimension subdim = drawFS(g,metrics,left+indent,y+h,fs.delta(featureCode));
	  if (g != null)
	    g.drawString(feature, left, y+h+(subdim.height-lineh)/2+ascent);
	  maxw = Math.max(maxw,indent + subdim.width);
	  h += subdim.height + d;
	}
      }
      extent = new Dimension(maxw+2*d,h);
      if (g != null) {
	Rectangle r = new Rectangle(new Point(x,y),extent);
	if (isList)
	  drawAngles(g,2*d,r);
	else
	  drawBrackets(g,d,r);
      }
    }
    if (tag != 0) {
      if (g != null) {
	int tagy = y+(extent.height-lineh)/2;
	g.drawString(tagstr,x-tagwidth+d/2,tagy+ascent+d/2);
	g.drawRect(x-tagwidth,tagy,tagwidth-d-1,lineh-1);
      }
      extent.width += tagwidth;
    }
    return extent;
  }


}
